/*
 *  BusMP MP V1.11
 *
 * Two threads instead of main + 1 thread
 * Line 601 if (ramsize < 384) not 192 for 256 MB RAM as uses 2 * 128 MB
 * Line 473 remove one tab
 * Line 691 extra noThread() and first pass microsecs
 * Version 1.2 arrays reduced to 768 and 384 word increments inc32, inc16, inc8
 *    memoryBytes[6]=1024*16380 not 16384  memoryBytes[7]=1024*131070 not 131072
 *    to fit with 768 inc. Reset andsums after error - 8 September 07
 */

 
#define NAME "Test"
#define TITLE " Roy Longbottom's MP Bus Speed Program "
#define INITGUID

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <xmmintrin.h>
#include <emmintrin.h>
#include <time.h>
#include "asmtime.h"
#include "BusMP64.h"

//  ************** PROTOPTYPES *****************

BOOL FAR PASCAL AboutDlgProc( HWND, UINT, WPARAM, LPARAM);
BOOL InitApp(HINSTANCE, LPSTR, int);
LRESULT CALLBACK  WindowProc(HWND, UINT, WPARAM, LPARAM);                  
static void CleanUpAndPostQuit(void);
void displayWhatever(HWND);

BOOL  FAR PASCAL TestPanel( HWND, UINT, WPARAM, LPARAM);
BOOL  runAllTests();
void adjust();

void local_time();

DWORD WINAPI threadTest1 (LPVOID inParam1);
DWORD WINAPI threadTest2 (LPVOID inParam2);

void noThread();
void inc32words();
void inc16words();
void inc8words();
void inc4words();
void inc2words();
void inc1word();
void incSSE2();
void inc32wordsT();
void inc16wordsT();
void inc8wordsT();
void inc4wordsT();
void inc2wordsT();
void inc1wordT();
void incSSE2T();

// ************* GLOBAL VARIABLES **********


__int64    * array;
__int64    * array2;

int      * andarray;
int      * andarrayT;

int    sse2loopinc;
int    sse2loopincT;

__m128i * arraym;
__m128i * arraym2;
__m128i * andSSE2;
__m128i * andSSE2T;

int           memoryWords[20];
unsigned int memoryBytes[20];
double        memoryMB[20];
// unsigned int wordBytes = 4;
unsigned int wordBytes = 8;

// char   version[30] = "32 bit Version 1.2";
char   version[30] = "64 bit Version 1.2";
// char   assembler[80] = "  SSE2 Assembled with Microsoft ml.exe Version 6.15.8803";
char   assembler[80] = " SSE2 Assembled with Microsoft ml64.exe Version 8.00.40310.39";
// char   compiler[80] = " Via Microsoft 32-bit C/C++ Optimizing Compiler Version 13.10.3077 for 80x86";
char   compiler[80] = " Via Microsoft C/C++ Optimizing Compiler Version 14.00.40310.41 for AMD64";
// char mips[80] = " For 32 bit MIPS divide MB/Second by 4. SSE2 divide by 16 for 128 bit MIPS";   
char mips[80] = " For 64 bit MIPS divide MB/Second by 8. SSE2 divide by 16 for 128 bit MIPS";   
// char    windowText[256] = " Roy Longbottom's 32 bit Multi-Thread Bus Speed Test";
char    windowText[256] = " Roy Longbottom's 64 bit Multi-Thread Bus Speed Test";

BOOL bQuit = FALSE;
HWND   mainHWND;
HWND   paramHwnd = NULL;
HINSTANCE  MyInstance;

char    timeday[30];

char    writeMsg[200];
char    runMsg[200] = " ";
char    commandLine[1024] = "";

int  runStage = -2;
int  runStage2 = 0;
int  mbpersec[20][20];


BOOL  autoRun = FALSE;
BOOL   debug = FALSE;

char fileName[256] = "BusSpd.txt";

HANDLE hThreadHandle1;
HANDLE hThreadHandle2;
HANDLE hSema1;
HANDLE hSema2;
LPVOID inputParam1;
LPVOID inputParam2;
DWORD dwThreadId1;
DWORD dwThreadId2;

volatile double  threadCount2;
volatile double  threadCount1;
volatile int testToRun;
volatile int wordsToTest;
volatile int passes1;
volatile double secs1;
volatile double secs2;
int threadPasses = 10;

int passes1Loops[20][20];
int msecs1[30][20];
int passesa[20][20];
int passesb[20][20];

int loopx;
double  passCount;
 
__int64 andsum1;
__int64 andsum2;
int ipass;


double runSecs = 0.05;


void displayWhatever(HWND hwnd)
{
    if (paramHwnd)   DestroyWindow(paramHwnd);
    SetWindowText(mainHWND, windowText);        
    paramHwnd = CreateDialog( MyInstance,"TEST_PANEL", hwnd, (DLGPROC)TestPanel);
          
} // displayWhatever

/*
 * WinMain - initialization, message loop
 */
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow)
{
    MSG         msg;
    
    hPrevInstance = hPrevInstance;

    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;

    GetMessage( &msg, NULL, 0, 0 );

    if (autoRun)
    {
       if (!runAllTests())
       {
          CleanUpAndPostQuit();
       }
       if (!bQuit)
       {
          if (paramHwnd) DestroyWindow(paramHwnd);            
          CleanUpAndPostQuit();
       }
    }
    while(!bQuit)
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {     
           if (msg.message == WM_QUIT)
           {
                CleanUpAndPostQuit();
                break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return ((int)msg.wParam);
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else
        {
           // make sure we go to sleep if we have nothing else to do
           WaitMessage();
        }
    }
    return ((int) msg.wParam);
    
} /* WinMain */

/****************************************************************************/
/*                   Initialization and object creation                     */
/****************************************************************************/
/*
 * InitApp
 * Creates window
 */
BOOL
InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow)
{
    HWND hwnd;
    WNDCLASS wc;
    int   i, j, m;
    char *delims = {" ,"};
    char commands[20][256];
    char *p;
    char buf[256] = "";
  
    /*
     * set up and registers the window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "GenericIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = "GenericMenu";
    wc.lpszClassName = NAME;
    if (!RegisterClass(&wc))
        return FALSE;

    /*
     * Create the window
     */

    hwnd =
        CreateWindow(
            NAME,
            TITLE,
            WS_OVERLAPPEDWINDOW | WS_HSCROLL | WS_VSCROLL,
            0,
            0,
            640,
            480,
            NULL, 
            NULL, 
            hInstance, 
            NULL 
        );

    if (!hwnd) return FALSE;

    mainHWND = hwnd;
    strcpy(commandLine, strupr(cmdline));
   
    getDetails();
    
    i = 0;
    p = strtok(commandLine, delims);
    while (p != NULL)
    {
        strcpy(commands[i], p);
        strcat(buf, commands[i]);
        strcat(buf, " ");
        i = i + 1;
        if (i > 12) break;
        p = strtok(NULL, delims);
    }
    if (i > 0)
    {
        strcpy(commandLine, buf);
    }
    else
    {
        strcpy(commandLine, "None");
    }
    autoRun = FALSE;
    for (j=0; j<i; j++)
    {
        switch(commands[j][0])
        {            
            case 'A': // Auto Run
             autoRun = TRUE;
             break;            

            case 'D': // Debug
             debug = TRUE;
             break;            

            case 'L': // log file
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%s", fileName);
             }
             break;

            case 'P': // ThreadPasses
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%d", &threadPasses);
                 if (threadPasses < 1) threadPasses = 1;
             }
             break;

            case 'M': // milliseconds for runSecs
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%d", &m);
                 if (m < 1) m = 1;
                 runSecs = (double)m / 1000.0;
             }
             break;

        }
    }

    // Display the window

    ShowWindow(hwnd, cmdshow);    
    UpdateWindow(hwnd);

    displayWhatever(mainHWND);
    
    return TRUE;
    
}  // InitApp


/*
 * WindowProc - handle messages for the main application window
 */

LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{
       
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;
        
    case WM_INITMENUPOPUP:
        break;
        
    case WM_CREATE:
        break;

    case WM_DESTROY:
        CleanUpAndPostQuit();
        break;

    case WM_COMMAND:
        switch( LOWORD( wparam ) )
        {
        case MENU_ABOUT:
            DialogBox( MyInstance,"AboutBox", hwnd, (DLGPROC)AboutDlgProc );
            break;

       case MENU_RUNALL:
            if (!runAllTests()) CleanUpAndPostQuit();
            displayWhatever(mainHWND);                                     
            break;

        case MENU_EXIT:
            CleanUpAndPostQuit();
            break;
                
        }
        break;
   
        case WM_PAINT:
             break;
           
        case WM_KEYDOWN:
            break;

       case WM_ACTIVATE:
            break;
    } 
    return (DefWindowProc( hwnd, msg, wparam, lparam ));

} /* WindowProc */



/*
 * AboutDlgProc - processes messages for the about dialog.
 */
BOOL FAR PASCAL AboutDlgProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
{
    lparam = lparam;

    switch( msg ) {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND:
        if( LOWORD( wparam ) == IDOK ) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
        }
        break;
    }
    return( FALSE );

} /* AboutDlgProc */



  
void local_time()
{

    time_t time_of_day;
    time_of_day = time( NULL ); 
    strcpy (timeday, ctime(&time_of_day)); // timeday = date and time
    return;
}



/*
 * CleanUpAndPostQuit
 * Release all D3DRM objects, post a quit message and set the bQuit flag
 */
void
CleanUpAndPostQuit(void)
{
    bQuit = TRUE;
    return;
}


BOOL FAR PASCAL TestPanel( HWND hdlg, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    wparam = wparam;
    lparam = lparam;
    char     tab = 9;
    char   newline[4] = "  ";
//    int FAR  tabs[11] = {1, 33, 65, 97, 129, 161, 193, 225, 257, 289, 321};
    int FAR  tabs[11] = {33, 65, 97, 129, 161, 193, 225, 257, 265, 321, 333};
    hdlg = hdlg;
    int  i;

     switch( msg )
     {
       case WM_INITDIALOG:

        SendDlgItemMessage(hdlg, DATA_LIST, LB_SETTABSTOPS, (WPARAM) 11,
                                                 (LPARAM) (int FAR*) tabs);
        if (runStage < 1)
        {
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[1]);
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[2]);
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[3]);
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[5]);
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[6]);
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[7]);
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[8]);
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) newline);
            sprintf(writeMsg, " Results are saved in file busSpd.txt.");
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        }
        else
        {
            sprintf(writeMsg, " Kbytes%c Inc32w%c Inc16w%c Inc 8w%c Inc 4w%c Inc 2w%c Inc 1w%c  SSE2  Threads",
                                tab, tab,  tab, tab, tab, tab, tab);
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            for (i=0; i<runStage; i++)
            {
                sprintf(writeMsg, " %8d%c %8d%c %8d%c %8d%c %8d%c %8d%c %8d%c %8d%c %c1",        
                                   memoryBytes[i]/1024, tab,
                                   mbpersec[0][i], tab, mbpersec[1][i], tab,
                                   mbpersec[2][i], tab, mbpersec[3][i], tab,
                                   mbpersec[4][i], tab, mbpersec[5][i], tab,
                                   mbpersec[6][i], tab, tab);
                                    
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            }
            if (runStage2 == 0)
            {
            }
        }
        if (runStage2 > 0)
        {
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) newline);
            for (i=0; i<runStage2; i++)
            {
                sprintf(writeMsg, " %8d%c %8d%c %8d%c %8d%c %8d%c %8d%c %8d%c %8d%c %c2",        
                                   memoryBytes[i]/1024, tab,
                                   mbpersec[7][i], tab, mbpersec[8][i], tab,
                                   mbpersec[9][i], tab, mbpersec[10][i], tab,
                                   mbpersec[11][i], tab, mbpersec[12][i], tab,
                                   mbpersec[13][i], tab, tab);
                                    
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            }
        }
        sprintf(writeMsg, " %c %c %c %s", tab, tab, tab, runMsg);
        SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);
        return( TRUE );
                    
       case WM_COMMAND:
 
        if( HIWORD( wparam ) == LBN_SELCHANGE)
        {
             displayWhatever(mainHWND);
        }
        else if( HIWORD( wparam ) == BN_CLICKED)
        {
            switch (LOWORD (wparam))
            {
                case RUN_ALL:
                 PostMessage(mainHWND, WM_COMMAND, MENU_RUNALL, 0);
                 break;

                case EXIT_BUTTON:
                 PostMessage(mainHWND, WM_COMMAND, MENU_EXIT, 0);
                 break;
            }
            displayWhatever(mainHWND);
        }

        return( TRUE );         
     }
     return( FALSE );
    
} // TestPanel


BOOL  runAllTests()
{
   int i, p;
   int j, tt0, tt7;
   int endTest;
   
   double addressInc[10];
   FILE *outfile;
   
   local_time();
   outfile = fopen(fileName,"a+");
   if (outfile == NULL)
   {
       MessageBox(NULL, "Cannot open log file", fileName,
                             MB_ICONWARNING | MB_OK);
       return FALSE;
   }

   fprintf (outfile, "\n\n #####################################################################\n");
   fprintf (outfile, "      MP Bus Speed Test %s %s \n", version, timeday);
   fprintf (outfile, "%s\n", compiler);
   fprintf (outfile, "      %s\n\n", assembler);

   fprintf (outfile, " This version uses integer AND instructions to a single register, streaming\n");   
   fprintf (outfile, " data from caches or RAM. First test reads one word with a 32 word address\n");
   fprintf (outfile, " increment for the next word. That is 128 bytes with 32 bit words and 256\n");
   fprintf (outfile, " bytes with 64 bit words. Address increment reduces for following tests to\n");
   fprintf (outfile, " one word (ReadAll). Last test reads all 16 byte SSE2 data. With two\n");
   fprintf (outfile, " threads, each reads separate data, with both threads running the same\n");
   fprintf (outfile, " number of passes as before.\n\n");

   fprintf (outfile, "                    Copyright Roy Longbottom 2007\n\n");
   fprintf (outfile, "                  Part 1 - Single Thread MBytes/Second\n\n");
   fprintf (outfile, "   Kbytes Inc32wds Inc16wds  Inc8wds  Inc4wds  Inc2wds  ReadAll 128bSSE2\n\n");
        //             12345678 12345678 12345678 12345678 12345678 12345678 12345678 12345678 

   fflush(outfile);

   addressInc[0] = 32; // inc32words
   addressInc[1] = 16; // inc16words
   addressInc[2] = 8;  // inc8words
   addressInc[3] = 4;  // inc4words
   addressInc[4] = 2;  // inc2words
   addressInc[5] = 1;  // inc1word
   addressInc[6] = 1;  // inc all words incSSE2
   
   memoryBytes[0] = 1024 * 6;
   memoryBytes[1] = 1024 * 24;
   memoryBytes[2] = 1024 * 96;
   memoryBytes[3] = 1024 * 384;
   memoryBytes[4] = 1024 * 768;
   memoryBytes[5] = 1024 * 1536;
   memoryBytes[6] = 1024 * 16380;  // 16384;
   memoryBytes[7] = 1024 * 131070; //131072;

   for (i=0; i<20; i++)
   {
       for (j=0; j<20; j++)
       {
           mbpersec[i][j] = 0;
           passes1Loops[j][i] = 0;
       }
   }
   ipass = 8;
   if (ramsize < 384) ipass = 7;
   if (ramsize < 32 ) ipass = 6;
   sprintf(runMsg, " ------ Test Running ------");
   for (i=0; i<ipass; i++)
   {
       memoryWords[i]  = memoryBytes[i] / wordBytes;
       memoryMB[i] = (double)memoryBytes[i] / 1000000;        
   }
   array = (__int64 *)_aligned_malloc(memoryBytes[ipass-1], 16);
   if (array == NULL)
   {
        sprintf(writeMsg, "Cannot allocate memory");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        CleanUpAndPostQuit();
        return FALSE;
   }

   arraym  = (__m128i *)array;
   
   array2 = (__int64 *)_aligned_malloc(memoryBytes[ipass-1], 16);
   if (array2 == NULL)
   {
        sprintf(writeMsg, "Cannot allocate memory");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        _aligned_free(array);
        CleanUpAndPostQuit();
        return FALSE;
   }

   arraym2  = (__m128i *)array2;

   andarray = (int *)_aligned_malloc(16, 16);
   if (andarray == NULL)
   {
        sprintf(writeMsg, "Cannot allocate memory");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        _aligned_free(array2);
        _aligned_free(array);
        CleanUpAndPostQuit();
        return FALSE;
   }
   andSSE2 = (__m128i *)andarray;
     
   andarrayT = (int *)_aligned_malloc(16, 16);
   if (andarrayT == NULL)
   {
        sprintf(writeMsg, "Cannot allocate memory");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        _aligned_free(array2);
        _aligned_free(array);
        CleanUpAndPostQuit();
        return FALSE;
   }
   andSSE2T = (__m128i *)andarrayT;

   for (i=0; i<memoryWords[ipass-1]; i++) // arraym
   {
       array[i] = 0x5555555555555555;
       array2[i] = 0x5555555555555555;
   }

   for (j=0; j<4; j++) // andSSE2
   {
       andarray[j] = 0x55555555;
       andarrayT[j] = 0x55555555;
   }

   andsum1 = 0x5555555555555555;
   andsum2 = 0x5555555555555555;
   
   runStage = 0;
   runStage2 = 0;

   displayWhatever(mainHWND);

   endTest = 7;
   if (hasSSE2) endTest = 8;

   // No Thread
   
   for (i=0; i<ipass; i++)
   {
       wordsToTest = memoryWords[i];
       runStage = runStage + 1;

       for (testToRun=1; testToRun<endTest; testToRun++)
       {
           tt0 = testToRun - 1;
           passes1 = 1;
           noThread();
           for (loopx=0; loopx<10; loopx++)
           {
               start_time();
               noThread();
               end_time();
               if (loopx == 0)
               {
                   msecs1[tt0 + 14][i] = (int)(secs * 1000000);
               }
               adjust();
           }
           passes1Loops[i][tt0] = passes1;
           start_time();
           for (p=0; p<threadPasses; p++)
           {
               noThread();
           }
           end_time();
           
           mbpersec[tt0][i] = (int)((double)passes1 * (double)threadPasses * memoryMB[i] / secs / addressInc[tt0]);
           msecs1[tt0][i] = (int)(secs * 1000);
           passesa[tt0][i] = passes1;
           displayWhatever(mainHWND);
       }
      

       if (andsum1 != 0x5555555555555555)
       {
           sprintf(writeMsg, "Wrong result Expected 0x5555555555555555 Was 0x%16.16lx at %d KB", andsum1, memoryBytes[i]/1024);
           MessageBox(NULL, writeMsg, "DATA ERROR", MB_ICONWARNING | MB_OK);
           fprintf(outfile, " %s\n", writeMsg);
           andsum1 = 0x5555555555555555;
       }

       if (hasSSE2)
       {
           for (j=0; j<4; j++) // andSSE2
           {
               if (andarray[j]  != 0x55555555)
               {
                   sprintf(writeMsg, "Wrong result Expected 0x55555555 Was 0x%8.8lx at %d KB SSE2", andarray[j], memoryBytes[i]/1024);
                   MessageBox(NULL, writeMsg, "DATA ERROR", MB_ICONWARNING | MB_OK);
                   fprintf(outfile, " %s\n", writeMsg);
                   andarray[j] = 0x55555555;
               }
           }

       }
   }
   displayWhatever(mainHWND);   
   for (i=0; i<runStage; i++)
   {
       fprintf(outfile, " %8d %8d %8d %8d %8d %8d %8d %8d\n",        
                           memoryBytes[i]/1024,
                           mbpersec[0][i], mbpersec[1][i],
                           mbpersec[2][i], mbpersec[3][i],
                           mbpersec[4][i], mbpersec[5][i],
                           mbpersec[6][i]);
   }
   fprintf (outfile, "\n"); 
   fprintf (outfile, "               Part 2 - Two Threads Total MBytes/Second\n\n");   
   fprintf (outfile, "   Kbytes Inc32wds Inc16wds  Inc8wds  Inc4wds  Inc2wds  ReadAll 128bSSE2\n\n");
   fflush (outfile);

   testToRun = 0;
   wordsToTest = 0;

   // Two Threads
   
   for (i=0; i<ipass; i++)
   {
       runStage2 = runStage2 + 1;
       wordsToTest = memoryWords[i];

       for (testToRun=1; testToRun<endTest; testToRun++)
       {
           tt0 = testToRun - 1;
           tt7 = testToRun + 6;
           displayWhatever(mainHWND);

           passes1 = passes1Loops[i][tt0];
           threadCount1 = threadPasses;
           threadCount2 = threadPasses;

           hSema1 = CreateSemaphore(NULL, 0, 100, NULL);
           hSema2 = CreateSemaphore(NULL, 0, 100, NULL);
           hThreadHandle1 = CreateThread(NULL, 0, threadTest1, inputParam1, 0, &dwThreadId1);
           hThreadHandle2 = CreateThread(NULL, 0, threadTest2, inputParam2, 0, &dwThreadId2);
           SetThreadPriority(hThreadHandle1, THREAD_PRIORITY_NORMAL);
           SetThreadPriority(hThreadHandle2, THREAD_PRIORITY_NORMAL);

           start_time();
           ReleaseSemaphore(hSema1, 1, NULL);
           ReleaseSemaphore(hSema1, 1, NULL);
           WaitForSingleObject(hSema2, 50000);
           WaitForSingleObject(hSema2, 50000);            
    
           CloseHandle( hThreadHandle2 );
           CloseHandle( hThreadHandle1 );
           CloseHandle( hSema2 );
           CloseHandle( hSema1 );
              
           displayWhatever(mainHWND);
           mbpersec[tt7][i] = (int)((threadCount1 + threadCount2) * (double)passes1 * memoryMB[i] / secs / addressInc[tt0]);
           msecs1[tt7][i] = (int)threadCount1;
           passesa[tt7][i] = (int)(secs1 * 1000);
           passesb[tt7][i] = (int)(secs2 * 1000);
       }
 
       if (andsum1 != 0x5555555555555555)
       {
           sprintf(writeMsg, "Wrong result Thread 1 Expected 0x5555555555555555 Was 0x%16.16lx at %d KB", andsum1, memoryBytes[i]/1024);
           MessageBox(NULL, writeMsg, "DATA ERROR", MB_ICONWARNING | MB_OK);
           fprintf(outfile, " %s\n", writeMsg);
           andsum1 = 0x5555555555555555;
       }
       if (andsum2 != 0x5555555555555555)
       {
           sprintf(writeMsg, "Wrong result Thread 2 Expected 0x5555555555555555 Was 0x%16.16lx at %d KB", andsum1, memoryBytes[i]/1024);
           MessageBox(NULL, writeMsg, "DATA ERROR", MB_ICONWARNING | MB_OK);
           fprintf(outfile, " %s\n", writeMsg);
           andsum2 = 0x5555555555555555;
       }
       if (hasSSE2)
       {
           for (j=0; j<4; j++) // andSSE2
           {
               if (andarray[j]  != 0x55555555)
               {
                   sprintf(writeMsg, "Wrong result Thread 1 Expected 0x55555555 Was 0x%8.8lx at %d KB SSE2", andarray[j], memoryBytes[i]/1024);
                   MessageBox(NULL, writeMsg, "DATA ERROR", MB_ICONWARNING | MB_OK);
                   fprintf(outfile, " %s\n", writeMsg);
                   andarray[j] = 0x55555555;
               }
               if (andarrayT[j]  != 0x55555555)
               {
                   sprintf(writeMsg, "Wrong result Thread 2 Expected 0x55555555 Was 0x%8.8lx at %d KB SSE2", andarrayT[j], memoryBytes[i]/1024);
                   MessageBox(NULL, writeMsg, "DATA ERROR", MB_ICONWARNING | MB_OK);
                   fprintf(outfile, " %s\n", writeMsg);
                   andarrayT[j] = 0x55555555;
               }
           }
       }       
       displayWhatever(mainHWND);
   }
   
   sprintf(runMsg, " ------ Test Finished ------");
   displayWhatever(mainHWND);

   for (i=0; i<runStage; i++)
   {
       fprintf(outfile, " %8d %8d %8d %8d %8d %8d %8d %8d\n",        
                           memoryBytes[i]/1024,
                           mbpersec[7][i], mbpersec[8][i],
                           mbpersec[9][i], mbpersec[10][i],
                           mbpersec[11][i], mbpersec[12][i],
                           mbpersec[13][i]);
   }
   fprintf (outfile, "\n"); 
   fprintf (outfile, " %s\n\n", mips);

   if (debug)
   {
       fprintf (outfile, "calibrate pass 1 microsecs\n"); 
       for (i=0; i<runStage; i++)
       {
           fprintf(outfile, " %8d %8d %8d %8d %8d %8d %8d %8d\n",        
                               memoryBytes[i]/1024,
                               msecs1[14][i], msecs1[15][i],
                               msecs1[16][i], msecs1[17][i],
                               msecs1[18][i], msecs1[19][i],
                               msecs1[20][i]);
       }
       fprintf (outfile, "part 1 msecs\n"); 
       for (i=0; i<runStage; i++)
       {
           fprintf(outfile, " %8d %8d %8d %8d %8d %8d %8d %8d\n",        
                               memoryBytes[i]/1024,
                               msecs1[0][i], msecs1[1][i],
                               msecs1[2][i], msecs1[3][i],
                               msecs1[4][i], msecs1[5][i],
                               msecs1[6][i]);
       }
       fprintf (outfile, "part 1 passes\n"); 
       for (i=0; i<runStage; i++)
       {
           fprintf(outfile, " %8d %8d %8d %8d %8d %8d %8d %8d\n",        
                               memoryBytes[i]/1024,
                               passesa[0][i], passesa[1][i],
                               passesa[2][i], passesa[3][i],
                               passesa[4][i], passesa[5][i],
                               passesa[6][i]);
       }
       fprintf (outfile, "part 2 threadcount\n"); 
       for (i=0; i<runStage; i++)
       {
           fprintf(outfile, " %8d %8d %8d %8d %8d %8d %8d %8d\n",        
                               memoryBytes[i]/1024,
                               msecs1[7][i], msecs1[8][i],
                               msecs1[9][i], msecs1[10][i],
                               msecs1[11][i], msecs1[12][i],
                               msecs1[13][i]);
       }
       fprintf (outfile, "part 2 msecs Thread 1\n"); 
       for (i=0; i<runStage; i++)
       {
           fprintf(outfile, " %8d %8d %8d %8d %8d %8d %8d %8d\n",        
                               memoryBytes[i]/1024,
                               passesa[7][i], passesa[8][i],
                               passesa[9][i], passesa[10][i],
                               passesa[11][i], passesa[12][i],
                               passesa[13][i]);
       }
       fprintf (outfile, "part 2 msecs Thread 2\n"); 
       for (i=0; i<runStage; i++)
       {
           fprintf(outfile, " %8d %8d %8d %8d %8d %8d %8d %8d\n",        
                               memoryBytes[i]/1024,
                               passesb[7][i], passesb[8][i],
                               passesb[9][i], passesb[10][i],
                               passesb[11][i], passesb[12][i],
                               passesb[13][i]);
       }
   }
   for (i=0; i<9; i++)
   {
       fprintf(outfile, "%s\n", configdata[i]);
   }
   fprintf(outfile, "\n");
   fflush (outfile);
   fclose(outfile);

   testToRun = 0;
   wordsToTest = 0;

   Sleep(1000);

//   MessageBox(NULL, writeMsg, "After CloseHandle", MB_ICONWARNING | MB_OK);

   _aligned_free(andarrayT);  
   _aligned_free(andarray);  
   _aligned_free(array2);
   _aligned_free(array);

   displayWhatever(mainHWND);
   return TRUE;

} // runAllTests


void adjust()
{
   double runtime = runSecs;
   
   if (secs > runtime)
   {
       loopx = 11;
   }
   else if (secs < runtime / 100)
   {
       passes1 = passes1 * 100;
   }
   else if (secs > runtime / 2)
   {
       passes1 = passes1 * (int)(runtime / secs);
       loopx = 11;
   }
   else
   {
       passes1 = passes1 * (int)(runtime / secs);
   }
}

void noThread()
{
    if (testToRun == 1) inc32words();
    if (testToRun == 2) inc16words();
    if (testToRun == 3) inc8words();
    if (testToRun == 4) inc4words();
    if (testToRun == 5) inc2words();
    if (testToRun == 6) inc1word();
    if (testToRun == 7) incSSE2();   
    return;
}

DWORD WINAPI threadTest1 (LPVOID inParam1)
{
    int i;

    inParam1 = inParam1;
    
    WaitForSingleObject(hSema1, 50000);
    for (i=0; i<(int)threadCount1; i++)
    {
        if (testToRun == 1) inc32words();
        if (testToRun == 2) inc16words();
        if (testToRun == 3) inc8words();
        if (testToRun == 4) inc4words();
        if (testToRun == 5) inc2words();
        if (testToRun == 6) inc1word();
        if (testToRun == 7) incSSE2();
    }   
    end_time();
    secs1 = secs;
    ReleaseSemaphore(hSema2, 1, NULL);
    return 0;
}

DWORD WINAPI threadTest2 (LPVOID inParam2)
{
    int i;

    inParam2 = inParam2;
    
    WaitForSingleObject(hSema1, 50000);
    for (i=0; i<(int)threadCount2; i++)
    {
        if (testToRun == 1) inc32wordsT();
        if (testToRun == 2) inc16wordsT();
        if (testToRun == 3) inc8wordsT();
        if (testToRun == 4) inc4wordsT();
        if (testToRun == 5) inc2wordsT();
        if (testToRun == 6) inc1wordT();
        if (testToRun == 7) incSSE2T();
    }   
    end_time();
    secs2 = secs;
    ReleaseSemaphore(hSema2, 1, NULL);
    return 0;
}

void inc32words()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+768)
       {
           andsum1 = andsum1 & array[i     ] & array[i+32  ] & array[i+64  ] & array[i+96  ]
                             & array[i+128 ] & array[i+160 ] & array[i+192 ] & array[i+224 ]
                             & array[i+256 ] & array[i+288 ] & array[i+320 ] & array[i+352 ]
                             & array[i+384 ] & array[i+416 ] & array[i+448 ] & array[i+480 ]
                             & array[i+512 ] & array[i+544 ] & array[i+576 ] & array[i+608 ]
                             & array[i+640 ] & array[i+672 ] & array[i+704 ] & array[i+736 ];
       }
   }
}

void inc16words()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+768)
       {
           andsum1 = andsum1 & array[i    ] & array[i+16 ] & array[i+32 ] & array[i+48 ]
                             & array[i+64 ] & array[i+80 ] & array[i+96 ] & array[i+112]
                             & array[i+128] & array[i+144] & array[i+160] & array[i+176]
                             & array[i+192] & array[i+208] & array[i+224] & array[i+240]
                             & array[i+256] & array[i+272] & array[i+288] & array[i+304]
                             & array[i+320] & array[i+336] & array[i+352] & array[i+368]
                             & array[i+384] & array[i+400] & array[i+416] & array[i+432]
                             & array[i+448] & array[i+464] & array[i+480] & array[i+496]
                             & array[i+512] & array[i+528] & array[i+544] & array[i+560]
                             & array[i+576] & array[i+592] & array[i+608] & array[i+624]
                             & array[i+640] & array[i+656] & array[i+672] & array[i+688]
                             & array[i+704] & array[i+720] & array[i+736] & array[i+752];
       }
   }
}

void inc8words()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+384)
       {
           andsum1 = andsum1 & array[i    ] & array[i+8  ] & array[i+16 ] & array[i+24 ]
                             & array[i+32 ] & array[i+40 ] & array[i+48 ] & array[i+56 ]
                             & array[i+64 ] & array[i+72 ] & array[i+80 ] & array[i+88 ]
                             & array[i+96 ] & array[i+104] & array[i+112] & array[i+120]
                             & array[i+128] & array[i+136] & array[i+144] & array[i+152]
                             & array[i+160] & array[i+168] & array[i+176] & array[i+184]
                             & array[i+192] & array[i+200] & array[i+208] & array[i+216]
                             & array[i+224] & array[i+232] & array[i+240] & array[i+248]
                             & array[i+256] & array[i+264] & array[i+272] & array[i+280]
                             & array[i+288] & array[i+296] & array[i+304] & array[i+312]
                             & array[i+320] & array[i+328] & array[i+336] & array[i+344]
                             & array[i+352] & array[i+360] & array[i+368] & array[i+376];
       }
   }
}

void inc4words()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+256)
       {
           andsum1 = andsum1 & array[i    ] & array[i+4  ] & array[i+8  ] & array[i+12 ]
                             & array[i+16 ] & array[i+20 ] & array[i+24 ] & array[i+28 ]
                             & array[i+32 ] & array[i+36 ] & array[i+40 ] & array[i+44 ]
                             & array[i+48 ] & array[i+52 ] & array[i+56 ] & array[i+60 ]
                             & array[i+64 ] & array[i+68 ] & array[i+72 ] & array[i+76 ]
                             & array[i+80 ] & array[i+84 ] & array[i+88 ] & array[i+92 ]
                             & array[i+96 ] & array[i+100] & array[i+104] & array[i+108]
                             & array[i+112] & array[i+116] & array[i+120] & array[i+124]
                             & array[i+128] & array[i+132] & array[i+136] & array[i+140]
                             & array[i+144] & array[i+148] & array[i+152] & array[i+156]
                             & array[i+160] & array[i+164] & array[i+168] & array[i+172]
                             & array[i+176] & array[i+180] & array[i+184] & array[i+188]
                             & array[i+192] & array[i+196] & array[i+200] & array[i+204]
                             & array[i+208] & array[i+212] & array[i+216] & array[i+220]
                             & array[i+224] & array[i+228] & array[i+232] & array[i+236]
                             & array[i+240] & array[i+244] & array[i+248] & array[i+252];
       }
   }
}

void inc2words()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+128)
       {
           andsum1 = andsum1 & array[i    ] & array[i+2  ] & array[i+4  ] & array[i+6  ]
                             & array[i+8  ] & array[i+10 ] & array[i+12 ] & array[i+14 ]
                             & array[i+16 ] & array[i+18 ] & array[i+20 ] & array[i+22 ]
                             & array[i+24 ] & array[i+26 ] & array[i+28 ] & array[i+30 ]
                             & array[i+32 ] & array[i+34 ] & array[i+36 ] & array[i+38 ]
                             & array[i+40 ] & array[i+42 ] & array[i+44 ] & array[i+46 ]
                             & array[i+48 ] & array[i+50 ] & array[i+52 ] & array[i+54 ]
                             & array[i+56 ] & array[i+58 ] & array[i+60 ] & array[i+62 ]
                             & array[i+64 ] & array[i+66 ] & array[i+68 ] & array[i+70 ]
                             & array[i+72 ] & array[i+74 ] & array[i+76 ] & array[i+78 ]
                             & array[i+80 ] & array[i+82 ] & array[i+84 ] & array[i+86 ]
                             & array[i+88 ] & array[i+90 ] & array[i+92 ] & array[i+94 ]
                             & array[i+96 ] & array[i+98 ] & array[i+100] & array[i+102]
                             & array[i+104] & array[i+106] & array[i+108] & array[i+110]
                             & array[i+112] & array[i+114] & array[i+116] & array[i+118]
                             & array[i+120] & array[i+122] & array[i+124] & array[i+126];
       }
    }
}

void inc1word()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+64)
       {
           andsum1 = andsum1 & array[i   ] & array[i+1 ] & array[i+2 ] & array[i+3 ]
                             & array[i+4 ] & array[i+5 ] & array[i+6 ] & array[i+7 ]
                             & array[i+8 ] & array[i+9 ] & array[i+10] & array[i+11]
                             & array[i+12] & array[i+13] & array[i+14] & array[i+15]
                             & array[i+16] & array[i+17] & array[i+18] & array[i+19]
                             & array[i+20] & array[i+21] & array[i+22] & array[i+23]
                             & array[i+24] & array[i+25] & array[i+26] & array[i+27]
                             & array[i+28] & array[i+29] & array[i+30] & array[i+31]
                             & array[i+32] & array[i+33] & array[i+34] & array[i+35]
                             & array[i+36] & array[i+37] & array[i+38] & array[i+39]
                             & array[i+40] & array[i+41] & array[i+42] & array[i+43]
                             & array[i+44] & array[i+45] & array[i+46] & array[i+47]
                             & array[i+48] & array[i+49] & array[i+50] & array[i+51]
                             & array[i+52] & array[i+53] & array[i+54] & array[i+55]
                             & array[i+56] & array[i+57] & array[i+58] & array[i+59]
                             & array[i+60] & array[i+61] & array[i+62] & array[i+63];
       }
   }
}

void incSSE2()
{
   int j;
   sse2loopinc = wordsToTest * wordBytes / 1024;
   for(j=0; j<passes1; j++)
   {
       asmsse2();
   }
}


void inc32wordsT()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+768)
       {
           andsum2 = andsum2 & array2[i     ] & array2[i+32  ] & array2[i+64  ] & array2[i+96  ]
                             & array2[i+128 ] & array2[i+160 ] & array2[i+192 ] & array2[i+224 ]
                             & array2[i+256 ] & array2[i+288 ] & array2[i+320 ] & array2[i+352 ]
                             & array2[i+384 ] & array2[i+416 ] & array2[i+448 ] & array2[i+480 ]
                             & array2[i+512 ] & array2[i+544 ] & array2[i+576 ] & array2[i+608 ]
                             & array2[i+640 ] & array2[i+672 ] & array2[i+704 ] & array2[i+736 ];
       }
   }
}

void inc16wordsT()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+768)
       {
           andsum2 = andsum2 & array2[i    ] & array2[i+16 ] & array2[i+32 ] & array2[i+48 ]
                             & array2[i+64 ] & array2[i+80 ] & array2[i+96 ] & array2[i+112]
                             & array2[i+128] & array2[i+144] & array2[i+160] & array2[i+176]
                             & array2[i+192] & array2[i+208] & array2[i+224] & array2[i+240]
                             & array2[i+256] & array2[i+272] & array2[i+288] & array2[i+304]
                             & array2[i+320] & array2[i+336] & array2[i+352] & array2[i+368]
                             & array2[i+384] & array2[i+400] & array2[i+416] & array2[i+432]
                             & array2[i+448] & array2[i+464] & array2[i+480] & array2[i+496]
                             & array2[i+512] & array2[i+528] & array2[i+544] & array2[i+560]
                             & array2[i+576] & array2[i+592] & array2[i+608] & array2[i+624]
                             & array2[i+640] & array2[i+656] & array2[i+672] & array2[i+688]
                             & array2[i+704] & array2[i+720] & array2[i+736] & array2[i+752];
       }
   }
}

void inc8wordsT()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+384)
       {
           andsum2 = andsum2 & array2[i    ] & array2[i+8  ] & array2[i+16 ] & array2[i+24 ]
                             & array2[i+32 ] & array2[i+40 ] & array2[i+48 ] & array2[i+56 ]
                             & array2[i+64 ] & array2[i+72 ] & array2[i+80 ] & array2[i+88 ]
                             & array2[i+96 ] & array2[i+104] & array2[i+112] & array2[i+120]
                             & array2[i+128] & array2[i+136] & array2[i+144] & array2[i+152]
                             & array2[i+160] & array2[i+168] & array2[i+176] & array2[i+184]
                             & array2[i+192] & array2[i+200] & array2[i+208] & array2[i+216]
                             & array2[i+224] & array2[i+232] & array2[i+240] & array2[i+248]
                             & array2[i+256] & array2[i+264] & array2[i+272] & array2[i+280]
                             & array2[i+288] & array2[i+296] & array2[i+304] & array2[i+312]
                             & array2[i+320] & array2[i+328] & array2[i+336] & array2[i+344]
                             & array2[i+352] & array2[i+360] & array2[i+368] & array2[i+376];
       }
   }
}

void inc4wordsT()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+256)
       {
           andsum2 = andsum2 & array2[i    ] & array2[i+4  ] & array2[i+8  ] & array2[i+12 ]
                             & array2[i+16 ] & array2[i+20 ] & array2[i+24 ] & array2[i+28 ]
                             & array2[i+32 ] & array2[i+36 ] & array2[i+40 ] & array2[i+44 ]
                             & array2[i+48 ] & array2[i+52 ] & array2[i+56 ] & array2[i+60 ]
                             & array2[i+64 ] & array2[i+68 ] & array2[i+72 ] & array2[i+76 ]
                             & array2[i+80 ] & array2[i+84 ] & array2[i+88 ] & array2[i+92 ]
                             & array2[i+96 ] & array2[i+100] & array2[i+104] & array2[i+108]
                             & array2[i+112] & array2[i+116] & array2[i+120] & array2[i+124]
                             & array2[i+128] & array2[i+132] & array2[i+136] & array2[i+140]
                             & array2[i+144] & array2[i+148] & array2[i+152] & array2[i+156]
                             & array2[i+160] & array2[i+164] & array2[i+168] & array2[i+172]
                             & array2[i+176] & array2[i+180] & array2[i+184] & array2[i+188]
                             & array2[i+192] & array2[i+196] & array2[i+200] & array2[i+204]
                             & array2[i+208] & array2[i+212] & array2[i+216] & array2[i+220]
                             & array2[i+224] & array2[i+228] & array2[i+232] & array2[i+236]
                             & array2[i+240] & array2[i+244] & array2[i+248] & array2[i+252];
       }
   }
}

void inc2wordsT()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+128)
       {
           andsum2 = andsum2 & array2[i    ] & array2[i+2  ] & array2[i+4  ] & array2[i+6  ]
                             & array2[i+8  ] & array2[i+10 ] & array2[i+12 ] & array2[i+14 ]
                             & array2[i+16 ] & array2[i+18 ] & array2[i+20 ] & array2[i+22 ]
                             & array2[i+24 ] & array2[i+26 ] & array2[i+28 ] & array2[i+30 ]
                             & array2[i+32 ] & array2[i+34 ] & array2[i+36 ] & array2[i+38 ]
                             & array2[i+40 ] & array2[i+42 ] & array2[i+44 ] & array2[i+46 ]
                             & array2[i+48 ] & array2[i+50 ] & array2[i+52 ] & array2[i+54 ]
                             & array2[i+56 ] & array2[i+58 ] & array2[i+60 ] & array2[i+62 ]
                             & array2[i+64 ] & array2[i+66 ] & array2[i+68 ] & array2[i+70 ]
                             & array2[i+72 ] & array2[i+74 ] & array2[i+76 ] & array2[i+78 ]
                             & array2[i+80 ] & array2[i+82 ] & array2[i+84 ] & array2[i+86 ]
                             & array2[i+88 ] & array2[i+90 ] & array2[i+92 ] & array2[i+94 ]
                             & array2[i+96 ] & array2[i+98 ] & array2[i+100] & array2[i+102]
                             & array2[i+104] & array2[i+106] & array2[i+108] & array2[i+110]
                             & array2[i+112] & array2[i+114] & array2[i+116] & array2[i+118]
                             & array2[i+120] & array2[i+122] & array2[i+124] & array2[i+126];
       }
   }
}

void inc1wordT()
{
   int i, j;

   for(j=0; j<passes1; j++)
   {
       for (i=0; i<wordsToTest; i=i+64)
       {
           andsum2 = andsum2 & array2[i   ] & array2[i+1 ] & array2[i+2 ] & array2[i+3 ]
                             & array2[i+4 ] & array2[i+5 ] & array2[i+6 ] & array2[i+7 ]
                             & array2[i+8 ] & array2[i+9 ] & array2[i+10] & array2[i+11]
                             & array2[i+12] & array2[i+13] & array2[i+14] & array2[i+15]
                             & array2[i+16] & array2[i+17] & array2[i+18] & array2[i+19]
                             & array2[i+20] & array2[i+21] & array2[i+22] & array2[i+23]
                             & array2[i+24] & array2[i+25] & array2[i+26] & array2[i+27]
                             & array2[i+28] & array2[i+29] & array2[i+30] & array2[i+31]
                             & array2[i+32] & array2[i+33] & array2[i+34] & array2[i+35]
                             & array2[i+36] & array2[i+37] & array2[i+38] & array2[i+39]
                             & array2[i+40] & array2[i+41] & array2[i+42] & array2[i+43]
                             & array2[i+44] & array2[i+45] & array2[i+46] & array2[i+47]
                             & array2[i+48] & array2[i+49] & array2[i+50] & array2[i+51]
                             & array2[i+52] & array2[i+53] & array2[i+54] & array2[i+55]
                             & array2[i+56] & array2[i+57] & array2[i+58] & array2[i+59]
                             & array2[i+60] & array2[i+61] & array2[i+62] & array2[i+63];
       }
   }
}


void incSSE2T()
{
   int j;

   sse2loopincT = wordsToTest * wordBytes / 1024;
   for(j=0; j<passes1; j++)
   {
       asmsse2T();
   }
}

