/*  
 *  CPUID Win64 Benchmark Version 1.0 Copyright (c) Roy Longbottom 2005 
 *
 * 64 bits        
 * counta
 * intCounta
 * icounta
 * totalI1a
 * totalI2a
 *
 */
 
#define NAME "Test"
#define TITLE " Roy Longbottom's Windows 64 CPU ID Program "
#define INITGUID

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <xmmintrin.h>
#include <emmintrin.h>
#include <time.h>
#include "asmtime.h"
#include "CpuID64.h"

//  ************** PROTOPTYPES *****************

BOOL FAR PASCAL AboutDlgProc( HWND, UINT, WPARAM, LPARAM);
BOOL InitApp(HINSTANCE, LPSTR, int);
LRESULT CALLBACK  WindowProc(HWND, UINT, WPARAM, LPARAM);                  
static void CleanUpAndPostQuit(void);
static BOOL GetFileName();
void displayWhatever(HWND);

BOOL  FAR PASCAL TestPanel( HWND, UINT, WPARAM, LPARAM);
BOOL  runAllTests();


void local_time();

BOOL saveData();

// ************* GLOBAL VARIABLES **********

char   currentDir[MAX_PATH];

unsigned int intCount;
unsigned int icount;
__int64 intCounta;
__int64 icounta;



double * dsum;
float  * ssum1;
int    * isum1;
 

// char   version[30] = "32 bit Version 3.0";
char   version[30] = "64 bit Version 1.0";
// char   assembler[70] = "    Assembled with Microsoft ml.exe Version 6.15.8803";
char   assembler[70] = " Assembled with Microsoft ml64.exe Version 8.00.40310.39";

BOOL bQuit = FALSE;

HWND   mainHWND;
HWND   paramHwnd = NULL;
HINSTANCE  MyInstance;

char    timeday[30];


char    writeMsg[200];
char    windowText[256];
char    commandLine[1024] = "";


int  runStage = -2;

int  mipsReg[4];
int  mipsaReg[4];

int  SSEmflops[4];
int  SSE2mflops[4];
int  SSE2mips[4];
int  SSE2mips2[4];

BOOL  correctInt;
BOOL  correctInta;

BOOL  correctSSE;
BOOL  correctSSE2;
BOOL  correctSSE2Int;
BOOL  correctSSE2Int2;
BOOL  allcorrect;

BOOL  autoRun = FALSE;

double        SSEcount;
double        SSE2count;
double        SSE2icount;
double        SSE2icount2;

char fileName[256] = "CPUInfo.txt";
BOOL  dataSaved = FALSE;


void displayWhatever(HWND hwnd)
{
    if (paramHwnd)   DestroyWindow(paramHwnd);
    sprintf(windowText, " Roy Longbottom's Windows 64 CPU ID and Speed Test");
    SetWindowText(mainHWND, windowText);        
    paramHwnd = CreateDialog( MyInstance,"TEST_PANEL", hwnd, (DLGPROC)TestPanel);
          
} // displayWhatever

/*
 * WinMain - initialization, message loop
 */
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow)
{
    MSG         msg;
    
    hPrevInstance = hPrevInstance;

    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;

    GetMessage( &msg, NULL, 0, 0 );
    if (!runAllTests())
    {
       CleanUpAndPostQuit();
    }
    else
    {
         displayWhatever(mainHWND);
    }
    if (autoRun)
    {
       if (!bQuit)
       {
          if (paramHwnd) DestroyWindow(paramHwnd);            
          saveData();
          CleanUpAndPostQuit();
       }
    }
    while(!bQuit)
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {     
           if (msg.message == WM_QUIT)
           {
                CleanUpAndPostQuit();
                break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return ((int)msg.wParam);
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else
        {
           // make sure we go to sleep if we have nothing else to do
           WaitMessage();
        }
    }
    return ((int) msg.wParam);
    
} /* WinMain */

/****************************************************************************/
/*                   Initialization and object creation                     */
/****************************************************************************/
/*
 * InitApp
 * Creates window
 */
BOOL
InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow)
{
    HWND hwnd;
    WNDCLASS wc;
    int   i, j;
    char *delims = {" ,"};
    char commands[20][256];
    char *p;
    char buf[256] = "";
  
    /*
     * set up and registers the window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "GenericIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = "GenericMenu";
    wc.lpszClassName = NAME;
    if (!RegisterClass(&wc))
        return FALSE;

    /*
     * Create the window
     */

    hwnd =
        CreateWindow(
            NAME,
            TITLE,
            WS_OVERLAPPEDWINDOW | WS_HSCROLL | WS_VSCROLL,
            0,
            0,
            640,
            480,
            NULL, 
            NULL, 
            hInstance, 
            NULL 
        );

    if (!hwnd) return FALSE;

    mainHWND = hwnd;
    strcpy(commandLine, strupr(cmdline));
   
    getDetails();
    
    i = 0;
    p = strtok(commandLine, delims);
    while (p != NULL)
    {
        strcpy(commands[i], p);
        strcat(buf, commands[i]);
        strcat(buf, " ");
        i = i + 1;
        if (i > 8) break;
        p = strtok(NULL, delims);
    }
    if (i > 0)
    {
        strcpy(commandLine, buf);
    }
    else
    {
        strcpy(commandLine, "None");
    }
    autoRun = FALSE;
    for (j=0; j<i; j++)
    {
        switch(commands[j][0])
        {            
            case 'A': // Auto Run
             autoRun = TRUE;
             break;            


            case 'L': // log file
             if (j < i -1)
             {
                 sscanf(commands[j+1], "%s", fileName);
             }
             break;
        }
    }

    // Display the window
    GetCurrentDirectory(MAX_PATH, currentDir);

    ShowWindow(hwnd, cmdshow);    
    UpdateWindow(hwnd);

    displayWhatever(mainHWND);
    
    return TRUE;
    
}  // InitApp


/*
 * WindowProc - handle messages for the main application window
 */

LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{
    BOOL doSave = FALSE;
    
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;
        
    case WM_INITMENUPOPUP:
        break;
        
    case WM_CREATE:
        break;

    case WM_DESTROY:
        CleanUpAndPostQuit();
        break;

    case WM_COMMAND:
        switch( LOWORD( wparam ) )
        {
        case MENU_ABOUT:
            DialogBox( MyInstance,"AboutBox", hwnd, (DLGPROC)AboutDlgProc );
            break;

       case MENU_RUNALL:
            if (!runAllTests()) CleanUpAndPostQuit();
            displayWhatever(mainHWND);                                     
            break;

       case MENU_SAVE:
            if (paramHwnd)   DestroyWindow(paramHwnd);            
            if( GetFileName())
            {
               saveData();
               SetCurrentDirectory(currentDir);
            }
            displayWhatever(mainHWND);
            break;

        case MENU_EXIT:
            if (!dataSaved)
            {
                 if (MessageBox(NULL, "Save last results ?", "Last Results Not Saved",
                 MB_ICONQUESTION | MB_YESNO) == IDYES) doSave = TRUE;
            }
            if (doSave)
            {
                if( GetFileName())
                {
                   saveData();
                   SetCurrentDirectory(currentDir);
                }
            }
            CleanUpAndPostQuit();
            break;
                
        }
        break;
   
        case WM_PAINT:
             break;
           
        case WM_KEYDOWN:
            break;

       case WM_ACTIVATE:
            break;
    } 
    return (DefWindowProc( hwnd, msg, wparam, lparam ));

} /* WindowProc */



/*
 * AboutDlgProc - processes messages for the about dialog.
 */
BOOL FAR PASCAL AboutDlgProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
{
    lparam = lparam;

    switch( msg ) {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND:
        if( LOWORD( wparam ) == IDOK ) {
            EndDialog( hwnd, TRUE );
            return( TRUE );
        }
        break;
    }
    return( FALSE );

} /* AboutDlgProc */



  
void local_time()
{

    time_t time_of_day;
    time_of_day = time( NULL ); 
    strcpy (timeday, ctime(&time_of_day)); // timeday = date and time
    return;
}



/*
 * CleanUpAndPostQuit
 * Release all D3DRM objects, post a quit message and set the bQuit flag
 */
void
CleanUpAndPostQuit(void)
{
    bQuit = TRUE;
    return;
}


BOOL FAR PASCAL TestPanel( HWND hdlg, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    wparam = wparam;
    lparam = lparam;
    char     tab = 9;
    char   lineSpace[2] = " ";
    int FAR  tabs[4] = {80, 125, 170, 215};
    hdlg = hdlg;

     switch( msg )
     {
       case WM_INITDIALOG:

        SendDlgItemMessage(hdlg, DATA_LIST, LB_SETTABSTOPS, (WPARAM) 4,
                                                 (LPARAM) (int FAR*) tabs);
        SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[1]);
        SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[2]);
        SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[3]);
        SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[5]);
        SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[6]);
        SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[7]);
        SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) configdata[8]);
        if (runStage == -1)
        {
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                            (LPARAM) lineSpace);
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                (LPARAM) " Integer Test Running");
        }
        if (runStage > -1)
        {
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) lineSpace);
            sprintf(writeMsg, " Speeds adding to %c 1 Register %c2 Registers"
                              " %c3 Registers %c4 Registers", tab, tab, tab, tab); 
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);

            sprintf(writeMsg, " 32 bit Integer MIPS %c%8d %c%8d %c%8d %c%8d",
                tab, mipsReg[0], tab, mipsReg[1], tab, mipsReg[2], tab, mipsReg[3]);    
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            if (runStage == 0)
            {
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                                (LPARAM) lineSpace);
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                    (LPARAM) " 64 bit Integer Test Running");
            }
        }
        if (runStage > 0)
        {

            sprintf(writeMsg, " 64 bit Integer MIPS %c%8d %c%8d %c%8d %c%8d",
                tab, mipsaReg[0], tab, mipsaReg[1], tab, mipsaReg[2], tab, mipsaReg[3]);    
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);

            if (runStage == 1)
            {
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                                (LPARAM) lineSpace);
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                    (LPARAM) " SSE2 32 Bit Integer Test Running");
            }
        }
        if (runStage > 1)
        {
            if (hasSSE2)
            {
                sprintf(writeMsg, " 32 bit SSE2 Int MIPS %c%8d %c%8d %c%8d %c%8d",    
                 tab, SSE2mips[0], tab, SSE2mips[1], tab, SSE2mips[2], tab, SSE2mips[3]);    
                 SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            }
            if (runStage == 2)
            {
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                                (LPARAM) lineSpace);
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                    (LPARAM) " SSE2 64 Bit Integer Test Running");
            }
        }
        if (runStage > 2)
        {
            if (hasSSE2)
            {
                sprintf(writeMsg, " 64 bit SSE2 Int MIPS %c%8d %c%8d %c%8d %c%8d",    
                 tab, SSE2mips2[0], tab, SSE2mips2[1], tab, SSE2mips2[2], tab, SSE2mips2[3]);    
                 SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            }
            if (runStage == 3)
            {
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                                (LPARAM) lineSpace);
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                    (LPARAM) " SSE Floating Point Test Running");
            }
        }
        if (runStage > 3)
        {
            if (hasSSE)
            {
                sprintf(writeMsg, " 32 bit SSE MFLOPS %c%8d %c%8d %c%8d %c%8d",    
                 tab, SSEmflops[0], tab, SSEmflops[1], tab, SSEmflops[2], tab, SSEmflops[3]);    
                 SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            }
            if (runStage == 4)
            {
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                                (LPARAM) lineSpace);
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,
                    (LPARAM) " SSE2 Floating Point Test Running");
            }
        }
        if (runStage > 4)
        {
            if (hasSSE2)
            {
                sprintf(writeMsg, " 64 bit SSE2 MFLOPS %c%8d %c%8d %c%8d %c%8d",    
                 tab, SSE2mflops[0], tab, SSE2mflops[1], tab, SSE2mflops[2], tab, SSE2mflops[3]);    
                 SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) writeMsg);
            }
        }
        if (runStage > 9)
        {
            allcorrect = TRUE;
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) lineSpace);
            if (!correctInt)
            {
                sprintf(writeMsg, " 32 bit Integer MIPS %c ERRORS INCORRECT ADDITIONS", tab);
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,(LPARAM) writeMsg);
                allcorrect = FALSE;
            }
            if (!correctInta)
            {
                sprintf(writeMsg, " 64 bit Integer MIPS %c ERRORS INCORRECT ADDITIONS", tab);
                SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,(LPARAM) writeMsg);
                allcorrect = FALSE;
            }
            if (hasSSE2)
            {
                if (!correctSSE2Int)
                {
                    sprintf(writeMsg, " SSE2 32 bit MIPS %c ERRORS INCORRECT ADDITIONS", tab);
                    SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,(LPARAM) writeMsg);
                    allcorrect = FALSE;
                }                
                if (!correctSSE2Int2)
                {
                    sprintf(writeMsg, " SSE2 64 bit MIPS %c ERRORS INCORRECT ADDITIONS", tab);
                    SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,(LPARAM) writeMsg);
                    allcorrect = FALSE;
                }                
            }
            if (hasSSE)
            {
                if (!correctSSE)
                {
                    sprintf(writeMsg, " SSE 32 bit MFLOPS %c ERRORS INCORRECT ADDITIONS", tab);
                    SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,(LPARAM) writeMsg);
                    allcorrect = FALSE;
                }                
            }
            if (hasSSE2)
            {
                if (!correctSSE2)
                {
                    sprintf(writeMsg, " SSE2 64 bit MFLOPS %c ERRORS INCORRECT ADDITIONS", tab);
                    SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,(LPARAM) writeMsg);
                    allcorrect = FALSE;
                }                
            }
            if (allcorrect)
            {
                    sprintf(writeMsg, " All instructions of r=r+1 correct results");
                    SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0,(LPARAM) writeMsg);
            }
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) lineSpace);
            SendDlgItemMessage(hdlg, DATA_LIST, LB_ADDSTRING, 0, (LPARAM) " Test Finished");
            SendDlgItemMessage(hdlg, DATA_LIST, LB_SETCURSEL, 17, 0L);
        }

        return( TRUE );
                    
       case WM_COMMAND:
 
        if( HIWORD( wparam ) == LBN_SELCHANGE)
        {
             displayWhatever(mainHWND);
        }
        else if( HIWORD( wparam ) == BN_CLICKED)
        {
            switch (LOWORD (wparam))
            {
                case RUN_ALL:
                 PostMessage(mainHWND, WM_COMMAND, MENU_RUNALL, 0);
                 break;

                case SAVE:
                 PostMessage(mainHWND, WM_COMMAND, MENU_SAVE, 0);
                 break;

                case EXIT_BUTTON:
                 PostMessage(mainHWND, WM_COMMAND, MENU_EXIT, 0);
                 break;
            }
            displayWhatever(mainHWND);
        }

        return( TRUE );         
     }
     return( FALSE );
    
} // TestPanel


BOOL  runAllTests()
{
   int i, mips, max, count;
   __int64 counta;
   int mflops;
   dataSaved = FALSE;
   unsigned int  totalI1  = 22000000;
   unsigned int  totalI2  = 23000000;

   __int64  totalI1a = 22000000;
   __int64  totalI2a = 23000000;

   double         totalS21 = 40000000;
   double         totalS22 = 42000000;
   double         totalS1  = 80000000;
   double         totalS2  = 84000000;
   double         totalS2i1  = 80000000;
   double         totalS2i2  = 84000000;
   double         totalS2i12  = 40000000;
   double         totalS2i22  = 42000000;

   intCount = 0;
   intCounta = 0;
   SSEcount = 0;
   SSE2count = 0;
   SSE2icount = 0;
   SSE2icount2 = 0;

   dsum = (double *)_aligned_malloc(64, 16);
   if (dsum == NULL)
   {
        sprintf(writeMsg, "Cannot allocate memory");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        CleanUpAndPostQuit();
        return FALSE;
   }        
   ssum1 = (float *)_aligned_malloc(64, 16);
   if (ssum1 == NULL)
   {
        sprintf(writeMsg, "Cannot allocate memory");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        _aligned_free(dsum);
        CleanUpAndPostQuit();
        return FALSE;
   }        
   isum1 = (int *)_aligned_malloc(64, 16);
   if (isum1 == NULL)
   {
        sprintf(writeMsg, "Cannot allocate memory");
        if (!autoRun) MessageBox(NULL, writeMsg, "ERROR WILL EXIT", MB_ICONWARNING | MB_OK);
        _aligned_free(ssum1);
        _aligned_free(dsum);
        CleanUpAndPostQuit();
        return FALSE;
   }        

   local_time();
   
   runStage = -1;
   displayWhatever(mainHWND);

   correctInt = TRUE;
   correctInta = TRUE;
   correctSSE = TRUE;
   correctSSE2 = TRUE;
   correctSSE2Int = TRUE;
   correctSSE2Int2 = TRUE;
   max = 0;

   for (i=0; i<10; i++)
   {
       count = intCount;
       start_time();
       mips1Reg();
       end_time();
       count = intCount - count;       
       mips = (int)((double)count / 1000000.0 / secs + 0.5);
       if(mips > max) max = mips;
   }       
   mipsReg[0] = max;

   max = 0;
   for (i=0; i<10; i++)
   {
       count = intCount;
       start_time();
       icount = intCount / 2;
       mips2Reg();
       end_time();
       count = intCount - count;       
       mips = (int)((double)count / 1000000.0 / secs + 0.5);
       if(mips > max) max = mips;
   }
   mipsReg[1] = max;

   max = 0;
   for (i=0; i<10; i++)
   {
       count = intCount;
       start_time();
       mips3Reg();
       end_time();
       count = intCount - count;       
       mips = (int)((double)count / 1000000.0 / secs + 0.5);
       if(mips > max) max = mips;
   }
   mipsReg[2] = max;
   
   max = 0;
   for (i=0; i<10; i++)
   {
       count = intCount;
       start_time();
       icount = intCount / 4;
       mips4Reg();
       end_time();
       count = intCount - count;      
       mips = (int)((double)count / 1000000.0 / secs + 0.5);
       if(mips > max) max = mips;
   }
   if(intCount != (totalI1 * 3 + totalI2) * 10) correctInt = FALSE;

   mipsReg[3] = max;

   runStage = 0;
   displayWhatever(mainHWND);

   max = 0;
   for (i=0; i<10; i++)
   {
       counta = intCounta;
       start_time();
       mipsa1Reg();
       end_time();
       counta = intCounta - counta;       
       mips = (int)((double)counta / 1000000.0 / secs + 0.5);
       if(mips > max) max = mips;
   }       
   mipsaReg[0] = max;

   max = 0;
   for (i=0; i<10; i++)
   {
       counta = intCounta;
       start_time();
       icounta = intCounta / 2;
       mipsa2Reg();
       end_time();
       counta = intCounta - counta;       
       mips = (int)((double)counta / 1000000.0 / secs + 0.5);
       if(mips > max) max = mips;
   }
   mipsaReg[1] = max;

   max = 0;
   for (i=0; i<10; i++)
   {
       counta = intCounta;
       start_time();
       mipsa3Reg();
       end_time();
       counta = intCounta - counta;       
       mips = (int)((double)counta / 1000000.0 / secs + 0.5);
       if(mips > max) max = mips;
   }
   mipsaReg[2] = max;
   
   max = 0;
   for (i=0; i<10; i++)
   {
       counta = intCounta;
       start_time();
       icounta = intCounta / 4;
       mipsa4Reg();
       end_time();
       counta = intCounta - counta;      
       mips = (int)((double)counta / 1000000.0 / secs + 0.5);
       if(mips > max) max = mips;
   }
   if(intCounta != (totalI1a * 3 + totalI2a) * 10) correctInta = FALSE;

   mipsaReg[3] = max;

   runStage = 1;
   displayWhatever(mainHWND);

   if (hasSSE2)
   {
       isum1[4] = 1;
       isum1[5] = 1;
       isum1[6] = 1;
       isum1[7] = 1;
       max = 0;
       for (i=0; i<10; i++)
       {    
           isum1[0] = 0;
           isum1[1] = 0;
           isum1[2] = 0;
           isum1[3] = 0;
           start_time();
           SSE2I1Reg();
           end_time();
           mips = (int)(totalS2i1 / 1000000.0 / secs + 0.5);
           if(mips > max) max = mips;
           SSE2icount = SSE2icount + (double) isum1[0] + (double) isum1[1]
                               + (double) isum1[2] + (double) isum1[3];
       }
       SSE2mips[0] = max;

       max = 0;
       for (i=0; i<10; i++)
       {
           isum1[0] = 0;
           isum1[1] = 0;
           isum1[2] = 0;
           isum1[3] = 0;
           start_time();
           SSE2I2Reg();
           end_time();
           mips = (int)(totalS2i1 / 1000000.0 / secs + 0.5);
           if(mips > max) max = mips;
           SSE2icount = SSE2icount + (double) isum1[0] + (double) isum1[1]
                               + (double) isum1[2] + (double) isum1[3];
       }
       SSE2mips[1] = max;

       max = 0;
       for (i=0; i<10; i++)
       {
           isum1[0] = 0;
           isum1[1] = 0;
           isum1[2] = 0;
           isum1[3] = 0;
           start_time();
           SSE2I3Reg();
           end_time();
           mips = (int)(totalS2i2 / 1000000.0 / secs + 0.5);
           if(mips > max) max = mips;
           SSE2icount = SSE2icount + (double) isum1[0] + (double) isum1[1]
                               + (double) isum1[2] + (double) isum1[3];
       }
       SSE2mips[2] = max;

       max = 0;
       for (i=0; i<10; i++)
       {
           isum1[0] = 0;
           isum1[1] = 0;
           isum1[2] = 0;
           isum1[3] = 0;
           start_time();
           SSE2I4Reg();
           end_time();
           mips = (int)(totalS2i1 / 1000000.0 / secs + 0.5);
           if(mips > max) max = mips;
           SSE2icount = SSE2icount + (double) isum1[0] + (double) isum1[1]
                               + (double) isum1[2] + (double) isum1[3];
       }
       SSE2mips[3] = max;
       if(SSE2icount != (totalS2i1 * 3 + totalS2i2) * 10) correctSSE2Int = FALSE;
   }

   runStage = 2;
   displayWhatever(mainHWND);

   if (hasSSE2)
   {
       isum1[4] = 1;
       isum1[5] = 0;
       isum1[6] = 1;
       isum1[7] = 0;
       max = 0;
       for (i=0; i<10; i++)
       {    
           isum1[0] = 0;
           isum1[1] = 0;
           isum1[2] = 0;
           isum1[3] = 0;
           start_time();
           SSE2I1Reg2();
           end_time();
           mips = (int)(totalS2i12 / 1000000.0 / secs + 0.5);
           if(mips > max) max = mips;
           SSE2icount2 = SSE2icount2 + (double) isum1[0] + (double) isum1[1]
                               + (double) isum1[2] + (double) isum1[3];
       }
       SSE2mips2[0] = max;

      max = 0;
       for (i=0; i<10; i++)
       {
           isum1[0] = 0;
           isum1[1] = 0;
           isum1[2] = 0;
           isum1[3] = 0;
           start_time();
           SSE2I2Reg2();
           end_time();
           mips = (int)(totalS2i12 / 1000000.0 / secs + 0.5);
           if(mips > max) max = mips;
           SSE2icount2 = SSE2icount2 + (double) isum1[0] + (double) isum1[1]
                               + (double) isum1[2] + (double) isum1[3];
       }
       SSE2mips2[1] = max;

       max = 0;
       for (i=0; i<10; i++)
       {
           isum1[0] = 0;
           isum1[1] = 0;
           isum1[2] = 0;
           isum1[3] = 0;
           start_time();
           SSE2I3Reg2();
           end_time();
           mips = (int)(totalS2i22 / 1000000.0 / secs + 0.5);
           if(mips > max) max = mips;
           SSE2icount2 = SSE2icount2 + (double) isum1[0] + (double) isum1[1]
                               + (double) isum1[2] + (double) isum1[3];
       }
       SSE2mips2[2] = max;

       max = 0;
       for (i=0; i<10; i++)
       {
           isum1[0] = 0;
           isum1[1] = 0;
           isum1[2] = 0;
           isum1[3] = 0;
           start_time();
           SSE2I4Reg2();
           end_time();
           mips = (int)(totalS2i12 / 1000000.0 / secs + 0.5);
           if(mips > max) max = mips;
           SSE2icount2 = SSE2icount2 + (double) isum1[0] + (double) isum1[1]
                               + (double) isum1[2] + (double) isum1[3];
       }
       SSE2mips2[3] = max;
       if(SSE2icount2 != (totalS2i12 * 3 + totalS2i22) * 10) correctSSE2Int2 = FALSE;
   }
   
   runStage = 3;
   displayWhatever(mainHWND);

   if (hasSSE)
   {
       ssum1[4] = 1;
       ssum1[5] = 1;
       ssum1[6] = 1;
       ssum1[7] = 1;
       max = 0;
       for (i=0; i<10; i++)
       {    
           ssum1[0] = 0;
           ssum1[1] = 0;
           ssum1[2] = 0;
           ssum1[3] = 0;
           start_time();
           SSE1Reg();
           end_time();
           mflops = (int)(totalS1 / 1000000.0 / secs + 0.5);
           if(mflops > max) max = mflops;
           SSEcount = SSEcount + (double) ssum1[0] + (double) ssum1[1]
                               + (double) ssum1[2] + (double) ssum1[3];
       }
       SSEmflops[0] = max;

      max = 0;
       for (i=0; i<10; i++)
       {
           ssum1[0] = 0;
           ssum1[1] = 0;
           ssum1[2] = 0;
           ssum1[3] = 0;
           start_time();
           SSE2Reg();
           end_time();
           mflops = (int)(totalS1 / 1000000.0 / secs + 0.5);
           if(mflops > max) max = mflops;
           SSEcount = SSEcount + (double) ssum1[0] + (double) ssum1[1]
                               + (double) ssum1[2] + (double) ssum1[3];
       }
       SSEmflops[1] = max;

       max = 0;
       for (i=0; i<10; i++)
       {
           ssum1[0] = 0;
           ssum1[1] = 0;
           ssum1[2] = 0;
           ssum1[3] = 0;
           start_time();
           SSE3Reg();
           end_time();
           mflops = (int)(totalS2 / 1000000.0 / secs + 0.5);
           if(mflops > max) max = mflops;
           SSEcount = SSEcount + (double) ssum1[0] + (double) ssum1[1]
                               + (double) ssum1[2] + (double) ssum1[3];
       }
       SSEmflops[2] = max;

       max = 0;
       for (i=0; i<10; i++)
       {
           ssum1[0] = 0;
           ssum1[1] = 0;
           ssum1[2] = 0;
           ssum1[3] = 0;
           start_time();
           SSE4Reg();
           end_time();
           mflops = (int)(totalS1 / 1000000.0 / secs + 0.5);
           if(mflops > max) max = mflops;
           SSEcount = SSEcount + (double) ssum1[0] + (double) ssum1[1]
                               + (double) ssum1[2] + (double) ssum1[3];
       }
       SSEmflops[3] = max;
       if(SSEcount != (totalS1 * 3 + totalS2) * 10) correctSSE = FALSE;
   }

   runStage = 4;
   displayWhatever(mainHWND);

   if (hasSSE2)
   {
       dsum[2] = 1;
       dsum[3] = 1;
       max = 0;
       for (i=0; i<10; i++)
       {    
           dsum[0] = 0;
           dsum[1] = 0;
           start_time();
           SSE21Reg();
           end_time();
           mflops = (int)(totalS21 / 1000000.0 / secs + 0.5);
           if(mflops > max) max = mflops;
           SSE2count = SSE2count + dsum[0] + dsum[1];
       }
       SSE2mflops[0] = max;

      max = 0;
       for (i=0; i<10; i++)
       {
           dsum[0] = 0;
           dsum[1] = 0;
           start_time();
           SSE22Reg();
           end_time();
           mflops = (int)(totalS21 / 1000000.0 / secs + 0.5);
           if(mflops > max) max = mflops;
           SSE2count = SSE2count + dsum[0] + dsum[1];
       }
       SSE2mflops[1] = max;

       max = 0;
       for (i=0; i<10; i++)
       {
           dsum[0] = 0;
           dsum[1] = 0;
           start_time();
           SSE23Reg();
           end_time();
           mflops = (int)(totalS22 / 1000000.0 / secs + 0.5);
           if(mflops > max) max = mflops;
           SSE2count = SSE2count + dsum[0] + dsum[1];
       }
       SSE2mflops[2] = max;

       max = 0;
       for (i=0; i<10; i++)
       {
           dsum[0] = 0;
           dsum[1] = 0;
           start_time();
           SSE24Reg();
           end_time();
           mflops = (int)(totalS21 / 1000000.0 / secs + 0.5);
           if(mflops > max) max = mflops;
           SSE2count = SSE2count + dsum[0] + dsum[1];
       }
       SSE2mflops[3] = max;
       if(SSE2count != (totalS21 * 3 + totalS22) * 10) correctSSE2 = FALSE;
   }
   runStage = 5;
   displayWhatever(mainHWND);

   runStage = 10;
   displayWhatever(mainHWND);
   _aligned_free(isum1);
   _aligned_free(ssum1);
   _aligned_free(dsum);   
   return TRUE;

} // runAllTests

static BOOL GetFileName()
{
    static char    filterList[] = "File (*.*)" \
                                        "\0" \
                                        "*.*" \
                                        "\0\0";
    OPENFILENAME        of;
    int                 rc;
    
    memset( &of, 0, sizeof( OPENFILENAME ) );
    of.lStructSize = sizeof( OPENFILENAME );
    of.hwndOwner = mainHWND;
    of.lpstrFilter = (LPSTR) filterList;
    of.lpstrDefExt = "";
    of.nFilterIndex = 1L;
    of.lpstrFile = fileName;
    of.nMaxFile = _MAX_PATH;
    of.lpstrTitle = NULL;
    of.lpstrInitialDir = currentDir;
    of.Flags = OFN_HIDEREADONLY;

    rc = GetSaveFileName( &of );
    strcpy(fileName, of.lpstrFile);
   
    return( rc );
    
} // GetFileName

BOOL saveData()
{
     FILE *outfile;
     int i;

     outfile = fopen(fileName,"a+");
     if (outfile == NULL)
     {
         MessageBox(NULL, "Cannot open log file", fileName,
                                 MB_ICONWARNING | MB_OK);
         return FALSE;
     }
     fprintf (outfile, "\n\n #####################################################################\n");
     fprintf (outfile, "   CPU ID and Speed Test %s %s \n", version, timeday);
     fprintf (outfile, "      %s\n\n", assembler);
     fprintf (outfile, "                    Copyright Roy Longbottom 2005\n\n");

     for (i=0; i<9; i++)
     {
         fprintf(outfile, "%s\n", configdata[i]);
     }
     fprintf(outfile,"\n");

     fprintf(outfile, " Speeds adding to     1 Register  2 Registers  3 Registers  4 Registers\n\n");
     fprintf(outfile, " 32 bit Integer MIPS %8d %12d %12d %12d\n",
                                        mipsReg[0], mipsReg[1], mipsReg[2], mipsReg[3]);    
     fprintf(outfile, " 64 bit Integer MIPS %8d %12d %12d %12d\n",
                                        mipsaReg[0], mipsaReg[1], mipsaReg[2], mipsaReg[3]);    
     if (hasSSE2)
     {
          fprintf(outfile, " 32 bit SSE2 Int MIPS%8d %12d %12d %12d\n",    
                                     SSE2mips[0], SSE2mips[1], SSE2mips[2], SSE2mips[3]);    
          fprintf(outfile, " 64 bit SSE2 Int MIPS%8d %12d %12d %12d\n",    
                                    SSE2mips2[0], SSE2mips2[1], SSE2mips2[2], SSE2mips2[3]);
     }
     if (hasSSE)
     {
          fprintf(outfile, " 32 bit SSE MFLOPS   %8d %12d %12d %12d\n",    
                                     SSEmflops[0], SSEmflops[1], SSEmflops[2], SSEmflops[3]);    
     }
     if (hasSSE2)
     {
          fprintf(outfile, " 64 bit SSE2 MFLOPS  %8d %12d %12d %12d\n",    
                                     SSE2mflops[0], SSE2mflops[1], SSE2mflops[2], SSE2mflops[3]);    
     }
     fprintf(outfile, "\n");
     if (allcorrect)
     {
         fprintf(outfile, " All instructions of r=r+1 correct results\n");
     }
     else
     {
         if (!correctInt)
         {
             fprintf(outfile, " 32 bit Integer MIPS  ERRORS INCORRECT ADDITIONS\n");
         }
         if (!correctInta)
         {
             fprintf(outfile, " 64 bit Integer MIPS  ERRORS INCORRECT ADDITIONS\n");
         }
         if (hasSSE2)
         {
                if (!correctSSE2Int)
                {
                    fprintf(outfile, " SSE2 32 bit MIPS %c ERRORS INCORRECT ADDITIONS");
                }
                if (!correctSSE2Int2)
                {
                    fprintf(outfile, " SSE2 64 bit MIPS %c ERRORS INCORRECT ADDITIONS");
                }
         }
         if (hasSSE)
         {
             if (!correctSSE)
             {
                  fprintf(outfile, " 32 bit SSE MFLOPS    ERRORS INCORRECT ADDITIONS " );
                  fprintf(outfile, " Was %16.3f \n", SSEcount);
             }                
         }
         if (hasSSE2)
         {
             if (!correctSSE2)
             {
                  fprintf(outfile, " 64 bit SSE2 MFLOPS   ERRORS INCORRECT ADDITIONS " );
                  fprintf(outfile, " Was %16.3f \n", SSE2count);
             }                
         }    
     }
     fprintf(outfile, "\n\n");
     fflush (outfile);
     fclose(outfile);  
     dataSaved = TRUE;
     return TRUE;
}


