/*
 *     C/C++ Whetstone Benchmark Single or Double Precision
 *     
 *  #define Precision and Version not used
 *     
 *  Different compilers can produce different floating point numeric
 *  results, probably due to compiling instructions in a different
 *  sequence. As the program checks these, they may need to be changed.
 *  The log file indicates non-standard results and these values can
 *  be copied and pasted into this program. See // Values near the
 *  end of main(). These values are for optimised compiling and non-
 *  optimised results might be different.
 *
 *  Different compilers do not optimise the code in the same way.
 *  This can lead to wide variations in benchmark speeds. See results
 *  with MS6 compiler ID and compare with those from same CPUs from
 *  the Watcom compiler generated code.
 *
 *  calibrate changed as optimised out to give no numeric results.
 *  Removed from function calls, now global,
 */
 
 #include "asmtime.h"
 #include <math.h>       /* for sin, exp etc.           */
 #include <stdio.h>      /* standard I/O                */
 #include <conio.h>      /* for getch  - 2 occurrences  */ 
 #include <string.h>     /* for strcpy - 1 occurrence   */
 #include <stdlib.h>     /* for exit   - 1 occurrence   */
 #include <windows.h>
 #include <time.h>

// PRECISION
//    #define SPDP double
//    #define Precision "Double Precision MP SSE2"
//    #define SorD "2"
//    #define optX "4"

    #define SPDP float
    #define Precision "Single Precision MP SSE"
//    #define SorD "1"
    #define SorD "3"  // AMD64
    #define optX "3"

// VERSION
//    #define options   "Via Microsoft 32-bit C/C++ Optimizing Compiler Version 13.10.3077 for 80x86"
    #define options   "Via Microsoft C/C++ Optimizing Compiler Version 14.00.40310.41 for AMD64"

    #define note1     "Uses 2 threads second at THREAD_PRIORITY_BELOW_NORMAL. Code produced by"
    #define note2     "compiler is not necessarily the same for both threads. So speed can vary"
    #define note3     "between threads. Vax MIPS are over-inflated due to excessive optimisation."

  void whetstones();  
  void whetstone2();  
  void pa(SPDP e[4], SPDP t, SPDP t2);
  void pa2(SPDP e2[4], SPDP tt, SPDP tt2);
  void po(SPDP e1[4], int j, int k, int l);
  void p3(SPDP *x, SPDP *y, SPDP *z, SPDP t, SPDP t1, SPDP t2);
  void p32(SPDP *xx, SPDP *yy, SPDP *zz, SPDP tt, SPDP tt1, SPDP tt2);
  void pout(char title[22], float ops, int type, SPDP checknum,
                  SPDP time, int section, SPDP checknum2);
  void printIt(SPDP val);
  void local_time();
  BOOL StartThread(); 
  DWORD WINAPI CountLoop(LPVOID inParam);
  BOOL  WINAPI RunThread();
  
  static SPDP loop_time[9];
  static SPDP loop_mops[9];
  static SPDP loop_mflops[9];
  static SPDP loop_mops2[9];
  static SPDP loop_mflops2[9];
  static SPDP loop_mopst[9];
  static SPDP loop_mflopst[9];
  static SPDP TimeUsed;
  static SPDP mwips;
  static char  headings[9][18];
  static SPDP Check;
  static SPDP results[9];
  FILE    *outfile;
  char    timeday[30];

  int  calibrate = 1;
  int xtra = 1;
  int x100 = 100;
  
  HANDLE hThreadHandle;
  LPVOID inputParam;
  BOOL  goThread;
  volatile int  threadCount = 0;
  volatile int  mainCount = 0;
  int  test;
  SPDP e1[4];
  SPDP e2[4];
  int  n1 = 12*x100;
  int  n2 = 14*x100;
  int  n3 = 345*x100;
  int  n4 = 210*x100;
  int  n5 = 32*x100;
  int  n6 = 899*x100;
  int  n7 = 616*x100;
  int  n8 = 93*x100;
  int  n1mult = 10;
  SPDP tt =  (SPDP)0.49999975;
  SPDP tt0 = tt;        
  SPDP tt1 = (SPDP)0.50000025;
  SPDP tt2 = 2.0;
  SPDP xx,yy,zz;
  int  jj, kk, ll;              

  main()
  {       
       int count = 10;
       int endit;
       int i;
       int errors;
       int x100 = 100;
       int duration = 10;
       double gmean1;
       double gmean2;
       char expect[9][30];
       char was[30];

       getDetails();

       printf("\n %s Whetstone Benchmark \n\n", Precision);
       printf(" %s\n", options);
       printf(" %s\n", note1);
       printf(" %s\n", note2);
       printf(" %s\n\n", note3);
         
       local_time();
       
       outfile = fopen("Whets.txt","a+");
       if (outfile == NULL)
       {
           printf (" Cannot open results file \n\n");
           printf(" Press any key\n");
           endit = getch();
           exit (0);
       }
       fprintf (outfile, " ********************************************************\n\n");                     
       fprintf (outfile, " Whetstone %s Benchmark %s\n",
                                          Precision, timeday);
       fflush(outfile);

       fprintf (outfile, " %s\n", options);
       fprintf (outfile, " %s\n", note1);
       fprintf (outfile, " %s\n", note2);
       fprintf (outfile, " %s\n\n", note3);
       if (strtol(optX, NULL, 10) == 3 && hasSSE == 0)
       {
           fprintf (outfile, " Cannot run SSE Instructions Not Available\n\n");
           printf (" Cannot run SSE Instructions Not Available\n\n");
           printf(" Press any key\n");
           endit = getch();
       }
       else if (strtol(optX, NULL, 10) == 4 && hasSSE2 == 0)
       {
           fprintf (outfile, " Cannot run SSE2 Instructions Not Available\n\n");
           printf (" Cannot run SSE Instructions Not Available\n\n");
           printf(" Press any key\n");
           endit = getch();
       }
       else
       {
           goThread = TRUE;
           StartThread();    
           SetThreadPriority(hThreadHandle, THREAD_PRIORITY_BELOW_NORMAL);
           SuspendThread(hThreadHandle);
           printf("Calibrate\n");
           do
           {
                TimeUsed=0;
                
                whetstones();
                
                printf("%11.5f Seconds %10.0lf   Passes (x 100)\n",
                                         TimeUsed,(SPDP)(xtra));
                calibrate = calibrate + 1;
                count = count - 1;
                if (TimeUsed > 0.2)
                   count = 0;
                else
                   xtra = xtra * 5;
           }
           while (count > 0);
           
           if (TimeUsed > 0)
                         xtra = (int)((SPDP)(duration * xtra) / TimeUsed);
           if (xtra < 1) xtra = 1;
           
           calibrate = 0;
      
           printf("\nUse %d  passes (x 100)\n", xtra);
    
           printf("\n %s Whetstone Benchmark \n\n", Precision);
           printf(" %s\n\n", options);
    
           printf("\nLoop content                  Result              MFLOPS "
                                    "     MOPS   Seconds\n\n");
    
           TimeUsed=0;
           whetstones();
    
           printf("\nMWIPS            ");
           if (TimeUsed>0)
             mwips=(float)(xtra) * (float)(x100) / (10 * TimeUsed);
           else
             mwips = 0;
           
           printf("%39.3f%19.3f\n\n",mwips,TimeUsed);
         
           if (Check == 0)
              printf("Wrong answer  ");
                  
    
    
        //               Add results to output file Whets.txt
    
        gmean1 = pow((double)(loop_mflopst[1] * loop_mflopst[2] * loop_mflopst[6]), 1.0/3.0);
        gmean2 = pow((double)(loop_mopst[4] * loop_mopst[3] * loop_mopst[7]), 1.0/3.0) * 5.0;
                              
        fprintf (outfile, " MFLOPS    Vax  MWIPS MFLOPS MFLOPS MFLOPS    Cos    Exp  Fixpt     If  Equal\n");
        fprintf (outfile, "  Gmean   MIPS            1      2      3    MOPS   MOPS   MOPS   MOPS   MOPS\n");
        printIt((SPDP)gmean1);
        printIt((SPDP)gmean2);    
        printIt((SPDP)mwips);
        printIt((SPDP)loop_mflopst[1]);
        printIt((SPDP)loop_mflopst[2]);
        printIt((SPDP)loop_mflopst[6]);
        printIt((SPDP)loop_mopst[5]);
        printIt((SPDP)loop_mopst[8]);
        printIt((SPDP)loop_mopst[4]);
        printIt((SPDP)loop_mopst[3]);
        printIt((SPDP)loop_mopst[7]);    
        fprintf(outfile,"\n");

        fprintf (outfile, "  Thread 1           ");
        printIt((SPDP)loop_mflops[1]);
        printIt((SPDP)loop_mflops[2]);
        printIt((SPDP)loop_mflops[6]);
        printIt((SPDP)loop_mops[5]);
        printIt((SPDP)loop_mops[8]);
        printIt((SPDP)loop_mops[4]);
        printIt((SPDP)loop_mops[3]);
        printIt((SPDP)loop_mops[7]);    
        fprintf(outfile,"\n");
        fprintf (outfile, "  Thread 2           ");
        printIt((SPDP)loop_mflops2[1]);
        printIt((SPDP)loop_mflops2[2]);
        printIt((SPDP)loop_mflops2[6]);
        printIt((SPDP)loop_mops2[5]);
        printIt((SPDP)loop_mops2[8]);
        printIt((SPDP)loop_mops2[4]);
        printIt((SPDP)loop_mops2[3]);
        printIt((SPDP)loop_mops2[7]);    
        fprintf(outfile,"\n\n");
    
    /*
        // Values Watcom
        
        if (strtol(SorD, NULL, 10) == 1)  // SP
        {
            sprintf(expect[1], "%24.17f", -1.12475025653839100);
            sprintf(expect[2], "%24.17f", -1.12274754047393800);
            sprintf(expect[3], "%24.17f",  1.00000000000000000);
            sprintf(expect[4], "%24.17f", 12.00000000000000000);
            sprintf(expect[5], "%24.17f",  0.49904659390449520);
            sprintf(expect[6], "%24.17f",  0.99999988079071040);
            sprintf(expect[7], "%24.17f",  3.00000000000000000);
            sprintf(expect[8], "%24.17f",  0.75110864639282230);
        }
        else // DP
        {
            sprintf(expect[1], "%24.17f", -1.12398255667391900);
            sprintf(expect[2], "%24.17f", -1.12187079889284400);
            sprintf(expect[3], "%24.17f",  1.00000000000000000);
            sprintf(expect[4], "%24.17f", 12.00000000000000000);
            sprintf(expect[5], "%24.17f",  0.49902937281515140);
            sprintf(expect[6], "%24.17f",  0.99999987890802820);
            sprintf(expect[7], "%24.17f",  3.00000000000000000);
            sprintf(expect[8], "%24.17f",  0.75100163018457870);
        }
    
        // Values MS Visual C++
        
        if (strtol(SorD, NULL, 10) == 1)  // SP
        {
            sprintf(expect[1], "%24.17f", -1.12441420555114750);
            sprintf(expect[2], "%24.17f", -1.12241148948669430);
            sprintf(expect[3], "%24.17f",  1.00000000000000000);
            sprintf(expect[4], "%24.17f", 12.00000000000000000);
            sprintf(expect[5], "%24.17f",  0.49907428026199341);
            sprintf(expect[6], "%24.17f",  0.99999988079071040);
            sprintf(expect[7], "%24.17f",  3.00000000000000000);
            sprintf(expect[8], "%24.17f",  0.75095528364181519);
        }
        else // DP
        {
            sprintf(expect[1], "%24.17f", -1.12398255667391100);
            sprintf(expect[2], "%24.17f", -1.12187079889295080);
            sprintf(expect[3], "%24.17f",  1.00000000000000000);
            sprintf(expect[4], "%24.17f", 12.00000000000000000);
            sprintf(expect[5], "%24.17f",  0.49902937281515491);
            sprintf(expect[6], "%24.17f",  0.99999987890802811);
            sprintf(expect[7], "%24.17f",  3.00000000000000000);
            sprintf(expect[8], "%24.17f",  0.75100163018453769);
        }
    
    */
        // Values MS Visual C++ Toolkit 2003
        
        if (strtol(SorD, NULL, 10) == 1)  // SP
        {
            sprintf(expect[1], "%24.17f", -1.12457144260406490);
            if (strtol(optX, NULL, 10) == 3)
            sprintf(expect[1], "%24.17f", -1.12391686439514160);
            sprintf(expect[2], "%24.17f", -1.12256872653961180);
            if (strtol(optX, NULL, 10) == 3)
            sprintf(expect[2], "%24.17f", -1.12191414833068850);
            sprintf(expect[3], "%24.17f",  1.00000000000000000);
            sprintf(expect[4], "%24.17f", 12.00000000000000000);
            sprintf(expect[5], "%24.17f",  0.49904659390449524);
            sprintf(expect[6], "%24.17f",  0.99999988079071040);
            if (strtol(optX, NULL, 10) == 3)
            sprintf(expect[6], "%24.17f",  0.99999982118606567);            
            sprintf(expect[7], "%24.17f",  3.00000000000000000);
            sprintf(expect[8], "%24.17f",  0.75095528364181519);
            sprintf(expect[8], "%24.17f",  0.75110864639282227);
        }
        else if (strtol(SorD, NULL, 10) == 3) // AMD64 SP
        {
            sprintf(expect[1], "%24.17f", -1.12475013732910160);
            sprintf(expect[2], "%24.17f", -1.12274742126464840);
            sprintf(expect[3], "%24.17f",  1.00000000000000000);
            sprintf(expect[4], "%24.17f", 12.00000000000000000);
            sprintf(expect[5], "%24.17f",  0.49909302592277527);
            sprintf(expect[6], "%24.17f",  0.99999982118606567);            
            sprintf(expect[7], "%24.17f",  3.00000000000000000);
            sprintf(expect[8], "%24.17f",  0.75110626220703125);
        }
        else // DP
        {
            sprintf(expect[1], "%24.17f", -1.12398255667391100);
            sprintf(expect[2], "%24.17f", -1.12187079889295080);
            if (strtol(optX, NULL, 10) == 4)
            sprintf(expect[2], "%24.17f", -1.12187079889294510);
            sprintf(expect[3], "%24.17f",  1.00000000000000000);
            sprintf(expect[4], "%24.17f", 12.00000000000000000);
            sprintf(expect[5], "%24.17f",  0.49902937281515491);
            sprintf(expect[6], "%24.17f",  0.99999987890802811);
            sprintf(expect[7], "%24.17f",  3.00000000000000000);
            sprintf(expect[8], "%24.17f",  0.75100163018453769);
        }
        
    
        errors = 0;
        
        for (i=1; i<9; i++)
        {
            sprintf(was, "%24.17f", results[i]); 
    
            if (strcmp (expect[i], was)  != 0)
            {
                fprintf(outfile, " Test %d Non-standard result was %s instead of %s\n",
                                   i, was, expect[i]);  
                errors = errors + 1;
            }
        }
        if (errors == 0)
        {
            fprintf(outfile, " Numeric results were as expected\n");
        }
    
        fprintf(outfile, "\n");
        ResumeThread(hThreadHandle);
        goThread = FALSE;
        CloseHandle( hThreadHandle );
        
    }    
    for (i=0; i<9; i++)
    {
        fprintf(outfile, "%s\n", configdata[i]);
    }
    fprintf(outfile,"\n");

    fclose (outfile);
    Sleep(1000);
    return 0;
  }

    void whetstones()
    {
        int i;
        SPDP x,y,z;              
        int j,k,l;
//        SPDP e1[4];
        double timeb;
                        
        SPDP t =  (SPDP)0.49999975;
        SPDP t0 = t;        
        SPDP t1 = (SPDP)0.50000025;
        SPDP t2 = 2.0;
                
        Check=0.0;
       

        /* Section 1, Array elements */

        e1[0] = 1.0;
        e1[1] = -1.0;
        e1[2] = -1.0;
        e1[3] = -1.0;
        e2[0] = 1.0;
        e2[1] = -1.0;
        e2[2] = -1.0;
        e2[3] = -1.0;
        start_time();
        test = 1;
        mainCount = 0;
        threadCount = 0;
        
        do
        {
            if (calibrate == 0)
            {
                ResumeThread(hThreadHandle);
                SetThreadPriority(hThreadHandle, THREAD_PRIORITY_BELOW_NORMAL);
            }
            mainCount = mainCount + 1;
            for(i=0; i<n1*n1mult; i++)
            {
                  e1[0] = (e1[0] + e1[1] + e1[2] - e1[3]) * t;
                  e1[1] = (e1[0] + e1[1] - e1[2] + e1[3]) * t;
                  e1[2] = (e1[0] - e1[1] + e1[2] + e1[3]) * t;
                  e1[3] = (-e1[0] + e1[1] + e1[2] + e1[3]) * t;
            }
            t = (SPDP)1.0 - t;
            if (calibrate == 0)
            {
                SuspendThread(hThreadHandle);
                tt = t0;
            }
        }
        while ((mainCount + threadCount) < (xtra));
        t =  t0;                    
        
        end_time();
        
        timeb = secs/(double)(n1mult);
        pout("N1 floating point\0",(float)(n1*16)*(float)(mainCount),
                             1,e1[3],(SPDP)timeb,1, e2[3]);

        /* Section 2, Array as parameter */

        start_time();
        test = 2;
        mainCount = 0;
        threadCount = 0;
        
        do
        {
            if (calibrate == 0)
            {
                ResumeThread(hThreadHandle);
                SetThreadPriority(hThreadHandle, THREAD_PRIORITY_BELOW_NORMAL);
            }
            mainCount = mainCount + 1;
            for(i=0; i<n2; i++)
            {
                 pa(e1,t,t2);
            }
            t = (SPDP)1.0 - t;
            if (calibrate == 0)
            {
                SuspendThread(hThreadHandle);
                tt = t0;
            }        
        }
        while ((mainCount + threadCount) < (xtra));
        
        t =  t0;
        end_time();
        timeb = secs;
        pout("N2 floating point\0",(float)(n2*96)*(float)(mainCount),
                             1,e1[3],(SPDP)timeb,2, e2[3]);

        /* Section 3, Conditional jumps */
        j = 1;
        jj = 1;
        start_time();
        test = 3;
        mainCount = 0;
        threadCount = 0;
        
        do
        {
            if (calibrate == 0)
            {
                ResumeThread(hThreadHandle);
                SetThreadPriority(hThreadHandle, THREAD_PRIORITY_BELOW_NORMAL);
            }
            mainCount = mainCount + 1;
            for(i=0; i<n3; i++)
            {
                 if(j==1)       j = 2;
                 else           j = 3;
                 if(j>2)        j = 0;
                 else           j = 1;
                 if(j<1)        j = 1;
                 else           j = 0;
            }
            if (calibrate == 0)
            {
                SuspendThread(hThreadHandle);
            }        
        }
        while ((mainCount + threadCount) < (xtra));
        end_time();
        timeb = secs;
        pout("N3 if then else  \0",(float)(n3*3)*(float)(mainCount),
                        2,(SPDP)(j),(SPDP)timeb,3, (SPDP)(jj));

        /* Section 4, Integer arithmetic */
        j = 1;
        k = 2;
        l = 3;
        jj = 1;
        kk = 2;
        ll = 3;
        start_time();
        test = 4;
        mainCount = 0;
        threadCount = 0;        
        do
        {
            if (calibrate == 0)
            {
                ResumeThread(hThreadHandle);
                SetThreadPriority(hThreadHandle, THREAD_PRIORITY_BELOW_NORMAL);
            }
            mainCount = mainCount + 1;
            for(i=0; i<n4; i++)
            {
                 j = j *(k-j)*(l-k);
                 k = l * k - (l-j) * k;
                 l = (l-k) * (k+j);
                 e1[l-2] = (SPDP)(j + k + l);
                 e1[k-2] = (SPDP)(j * k * l);
            }
            if (calibrate == 0)
            {
                SuspendThread(hThreadHandle);
            }        
        }
        while ((mainCount + threadCount) < (xtra));
        end_time();
        timeb = secs;
        x = e1[0]+e1[1];
        xx = e2[0]+e2[1];
        pout("N4 fixed point   \0",(float)(n4*15)*(float)(mainCount),
                                 2,x,(SPDP)timeb,4, xx);
     
        /* Section 5, Trig functions */
        x = 0.5;
        y = 0.5;
        xx = 0.5;
        yy = 0.5;
        start_time();
        test = 5;
        mainCount = 0;
        threadCount = 0;        
        do
        {
            if (calibrate == 0)
            {
                ResumeThread(hThreadHandle);
                SetThreadPriority(hThreadHandle, THREAD_PRIORITY_BELOW_NORMAL);
            }
            mainCount = mainCount + 1;
            for(i=1; i<n5; i++)
            {
                 x = (SPDP)(t*atan(t2*sin(x)*cos(x)/(cos(x+y)+cos(x-y)-1.0)));
                 y = (SPDP)(t*atan(t2*sin(y)*cos(y)/(cos(x+y)+cos(x-y)-1.0)));
            }
            t = (SPDP)1.0 - t;
            if (calibrate == 0)
            {
                SuspendThread(hThreadHandle);
            }        
        }
        while ((mainCount + threadCount) < (xtra)); 
        t = t0;
        end_time();
        timeb = secs;
        pout("N5 sin,cos etc.  \0",(float)(n5*26)*(float)(mainCount),
                                 2,y,(SPDP)timeb,5,yy);
  
        /* Section 6, Procedure calls */
        x = 1.0;
        y = 1.0;
        z = 1.0;
        xx = 1.0;
        yy = 1.0;
        zz = 1.0;

        test = 6;
        mainCount = 0;
        threadCount = 0;
        start_time();
        do
        {
            if (calibrate == 0)
            {
                ResumeThread(hThreadHandle);
                SetThreadPriority(hThreadHandle, THREAD_PRIORITY_BELOW_NORMAL);
            }
            mainCount = mainCount + 1;
            for(i=0; i<n6; i++)
            {
                 p3(&x,&y,&z,t,t1,t2);
            }
            if (calibrate == 0)
            {
                SuspendThread(hThreadHandle);
            }        
        }
        while ((mainCount + threadCount) < (xtra));
        end_time();
        timeb = secs;
        pout("N6 floating point\0",(float)(n6*6)*(float)(mainCount),
                                1,z,(SPDP)timeb,6,zz);
  
        /* Section 7, Array refrences */
        j = 0;
        k = 1;
        l = 2;
        e1[0] = 1.0;
        e1[1] = 2.0;
        e1[2] = 3.0;
        jj = 0;
        kk = 1;
        ll = 2;
        e2[0] = 1.0;
        e2[1] = 2.0;
        e2[2] = 3.0;
        start_time();
        test = 7;
        mainCount = 0;
        threadCount = 0;
        start_time();
        do
        {
            if (calibrate == 0)
            {
                ResumeThread(hThreadHandle);
                SetThreadPriority(hThreadHandle, THREAD_PRIORITY_BELOW_NORMAL);
            }
            mainCount = mainCount + 1;
            for(i=0;i<n7;i++)
            {
                 po(e1,j,k,l);
            }
            if (calibrate == 0)
            {
                SuspendThread(hThreadHandle);
            }        
        }
        while ((mainCount + threadCount) < (xtra));        
        end_time();
        timeb = secs;
        pout("N7 assignments   \0",(float)(n7*3)*(float)(mainCount),
                            2,e1[2],(SPDP)timeb,7,e2[2]);
        
        /* Section 8, Standard functions */
        x = 0.75;
        xx = 0.75;
        tt1 = t1;
        start_time();
        test = 8;
        mainCount = 0;
        threadCount = 0;
        start_time();
        do
        {
            if (calibrate == 0)
            {
                ResumeThread(hThreadHandle);
                SetThreadPriority(hThreadHandle, THREAD_PRIORITY_BELOW_NORMAL);
            }
            mainCount = mainCount + 1;
            for(i=0; i<n8; i++)
            {
                 x = (SPDP)(sqrt(exp(log(x)/t1)));
            }
            if (calibrate == 0)
            {
                SuspendThread(hThreadHandle);
            }        
        }
        while ((mainCount + threadCount) < (xtra));        
        end_time();
        timeb = secs;
        pout("N8 exp,sqrt etc. \0",(float)(n8*4)*(float)(mainCount),
                                2,x,(SPDP)timeb,8,xx);

        return;
    }


    void pa(SPDP e[4], SPDP t, SPDP t2)
      {
         int j;
         for(j=0;j<6;j++)
            {
               e[0] = (e[0]+e[1]+e[2]-e[3])*t;
               e[1] = (e[0]+e[1]-e[2]+e[3])*t;
               e[2] = (e[0]-e[1]+e[2]+e[3])*t;
               e[3] = (-e[0]+e[1]+e[2]+e[3])/t2;
            }

         return;
      }

    void pa2(SPDP e[4], SPDP t, SPDP t2)
      {
         int j;
         for(j=0;j<6;j++)
            {
               e[0] = (e[0]+e[1]+e[2]-e[3])*t;
               e[1] = (e[0]+e[1]-e[2]+e[3])*t;
               e[2] = (e[0]-e[1]+e[2]+e[3])*t;
               e[3] = (-e[0]+e[1]+e[2]+e[3])/t2;
            }

         return;
      }

    void po(SPDP e1[4], int j, int k, int l)
      {
         e1[j] = e1[k];
         e1[k] = e1[l];
         e1[l] = e1[j];
         return;
      }

    void p3(SPDP *x, SPDP *y, SPDP *z, SPDP t, SPDP t1, SPDP t2)
      {
         *x = *y;
         *y = *z;
         *x = t * (*x + *y);
         *y = t1 * (*x + *y);
         *z = (*x + *y)/t2;
         return;
      }

    void p32(SPDP *xx, SPDP *yy, SPDP *zz, SPDP tt, SPDP tt1, SPDP tt2)
      {
         *xx = *yy;
         *yy = *zz;
         *xx = tt * (*xx + *yy);
         *yy = tt1 * (*xx + *yy);
         *zz = (*xx + *yy)/tt2;
         return;
      }

    void pout(char title[18], float ops, int type, SPDP checknum,
              SPDP time, int section, SPDP checknum2)
      {
        SPDP mops,mflops;

        Check = Check + checknum + checknum2;
        loop_time[section] = time;
        strcpy (headings[section],title);
        TimeUsed =  TimeUsed + time;
        if (calibrate == 1)     
        {
              results[section] = checknum;
        }
        if (calibrate == 0)
          {              
            printf("%s %24.17f    ",headings[section],results[section]);    
       
            if (type == 1)
             {
                if (time>0)
                 {
                    mflops = ops/(1000000L*time);
                 }
                else
                 {
                   mflops = 0;
                 }
                loop_mops[section] = 99999;
                loop_mflops[section] = mflops;
                loop_mflops2[section] = mflops * (SPDP)(threadCount) / (SPDP)(mainCount);
                printf(" %9.3f          %9.3f\n",
                 loop_mflops[section], loop_time[section]);                
                printf("Thread 2 Passes %8d Main Thread %8d  %9.3f          %9.3f\n",
                 threadCount, mainCount, loop_mflops2[section], loop_time[section]);                
                loop_mflopst[section] = loop_mflops2[section] + loop_mflops[section];
             }
            else
             {
                if (time>0)
                 {
                   mops = ops/(1000000L*time);
                 }
                else
                 {
                   mops = 0;
                 }
                loop_mops[section] = mops;
                loop_mops2[section] = mops * (SPDP)(threadCount) / (SPDP)(mainCount);
                loop_mflops[section] = 0;                 
                printf("           %9.3f%9.3f\n",
                 loop_mops[section], loop_time[section]);
                printf("Thread 2 Passes %8d Main Thread %8d            %9.3f%9.3f\n",
                 threadCount, mainCount, loop_mops2[section], loop_time[section]);
                loop_mopst[section] = loop_mops2[section] + loop_mops[section];
             }
          }
          
        return;
      }


void local_time()
{
    time_t time_of_day;
    time_of_day = time( NULL ); 
    strcpy (timeday, ctime(&time_of_day)); /* timeday = date and time */
    return;
}


void printIt(SPDP val)
{
    if (val > 100)
    {
        fprintf (outfile, "%7.0f", val);
    }
    else if (val > 10)
    {
        fprintf (outfile, "%7.1f", val);
    }
    else if (val > 1)
    {
        fprintf (outfile, "%7.2f", val);
    }
    else
    {
        fprintf (outfile, "%7.3f", val);
    }
}
        
BOOL StartThread()
{
    DWORD dwThreadId;

    hThreadHandle = CreateThread(
                        NULL,
                        0,
                        CountLoop,
                        inputParam,
                        0,
                        &dwThreadId
                        );
    if ( hThreadHandle == NULL ) 
    {
        return FALSE;
    }   
   return TRUE;
}

DWORD WINAPI CountLoop(LPVOID inParam)
{
    BOOL threadNeeded = TRUE;
    inParam = inParam;
    
    while (threadNeeded)
    {
        if (RunThread()) threadNeeded = FALSE;
        // FALSE only when Windows closes
    }
    return 0;
}

BOOL WINAPI RunThread()
{
    whetstone2(); 
    if (!goThread) 
    {
        return TRUE;
    }
    return FALSE; // exit here until Windows closes
}

void whetstone2()
{
    int i;

    threadCount = threadCount + 1;
    
    if (test == 1)
    {
        for(i=0; i<n1*n1mult; i++)
        {
              e2[0] = (e2[0] + e2[1] + e2[2] - e2[3]) * tt;
              e2[1] = (e2[0] + e2[1] - e2[2] + e2[3]) * tt;
              e2[2] = (e2[0] - e2[1] + e2[2] + e2[3]) * tt;
              e2[3] = (-e2[0] + e2[1] + e2[2] + e2[3]) * tt;
        }
        tt = (SPDP)1.0 - tt;
    }
    else if (test == 2)
    {
        for(i=0; i<n2; i++)
        {
             pa2(e2,tt,tt2);
        }
        tt = (SPDP)1.0 - tt;
    }
    else if (test == 3)
    {
        for(i=0; i<n3; i++)
        {
             if(jj==1)       jj = 2;
             else            jj = 3;
             if(jj>2)        jj = 0;
             else            jj = 1;
             if(jj<1)        jj = 1;
             else            jj = 0;
        }
    }
    else if (test == 4)
    {
        for(i=0; i<n4; i++)
        {
             jj = jj *(kk-jj)*(ll-kk);
             kk = ll * kk - (ll-jj) * kk;
             ll = (ll-kk) * (kk+jj);
             e2[ll-2] = (SPDP)(jj + kk + ll);
             e2[kk-2] = (SPDP)(jj * kk * ll);
        }
    }
    else if (test == 5)
    {
        for(i=1; i<n5; i++)
        {
             xx = (SPDP)(tt*atan(tt2*sin(xx)*cos(xx)/(cos(xx+yy)+cos(xx-yy)-1.0)));
             yy = (SPDP)(tt*atan(tt2*sin(yy)*cos(yy)/(cos(xx+yy)+cos(xx-yy)-1.0)));
        }
        tt = (SPDP)1.0 - tt;
    }
    else if (test == 6)
    {
        for(i=0; i<n6; i++)
        {
             p32(&xx,&yy,&zz,tt,tt1,tt2);
        }
    }
    else if (test == 7)
    {
        for(i=0;i<n7;i++)
        {
         e2[jj] = e2[kk];
         e2[kk] = e2[ll];
         e2[ll] = e2[jj];
        }
    }
    else if (test == 8)
    {
        for(i=0; i<n8; i++)
        {
             xx = (SPDP)(sqrt(exp(log(xx)/tt1)));
        }
    }
    if ((mainCount + threadCount) > (xtra - 2)) SetThreadPriority(hThreadHandle, THREAD_PRIORITY_IDLE);
}
